/*
 * pib.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib.h>

///////////////////////////////////////////////////////////////////////////////
int pib_create_ini_path(wchar_t **pathw, const wchar_t *namew)
{
  static const wchar_t PLUGINSW[]=L"Plugins";

  wchar_t *inidir=(wchar_t*)SendMessage(plugin_hwnd_main(),WM_WA_IPC,0,
      IPC_GETINIDIRECTORYW);
  size_t len=wcslen(inidir)
      +1u                 // '\\'
      +wcslen(namew)
      +1u                 // '\\'
      +wcslen(PLUGINSW)
      +4u;                // .ini

  ////////
  *pathw=malloc((len+1)*sizeof *pathw[0]);

  if (!*pathw) {
    DMESSAGE("allocating path");
    goto e_path;
  }

DVWRITELNW("\"%" PIB_PRIsw L"\"",inidir);
DVWRITELNW("\"%" PIB_PRIsw L"\"",PLUGINSW);
DVWRITELNW("\"%" PIB_PRIsw L"\"",namew);
  swprintf(*pathw,len+1,L"%" PIB_PRIsw L"\\%" PIB_PRIsw L"\\%"
			PIB_PRIsw L".ini", inidir,PLUGINSW,namew);
DVWRITELNW("\"%" PIB_PRIsw L"\"",*pathw);

  ////////
  return 0;
e_path:
  return -1;
}

void pib_destroy_ini_path(wchar_t **pathw)
{
  if (*pathw) {
    free(*pathw);
    *pathw=NULL;
  }
}

///////////////////////////////////////////////////////////////////////////////
HWND pib_hwnd_parent(void)
{
  /*
   * making the playlist editor the parent of our modless dialog let
   * appear our modless dialog always on top of Winamp. that's exactly
   * what we want to have.
   */
  HWND hwnd_main=plugin_hwnd_main();
  HWND hwnd_pe=(HWND)SendMessageW(hwnd_main,WM_WA_IPC,IPC_GETWND_PE,
      IPC_GETWND);

//DWRITELN("##########################################################");
//DVWRITELN("%p",GetParent(hwnd_pe));
//DWRITELN("##########################################################");
  return GetParent(hwnd_pe)?hwnd_pe:hwnd_main;
}
