/*
 * pib.h
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#ifndef __PIB_H__ // [
#define __PIB_H__

#if defined (UNICODE) // [
#define UNICODE
#endif // ]
#include <windows.h>
#include <commctrl.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <winamp/gen.h>
#include <pib_ipc.h>

///////////////////////////////////////////////////////////////////////////////
//#define DEBUG
#define PIB_CONTROL_DISABLED
//#define PIB_TIMER

/*
 * NOTE: avoid defining PIB_SEND_MESSAGE!
 */
//#define PIB_SEND_MESSAGE
/*
 * About SysLink Controls
 *   https://docs.microsoft.com/en-us/windows/win32/controls/syslink-overview
 */

#define PIB_IMAGE_BASE
#if defined (PIB_IMAGE_BASE) // [
EXTERN_C IMAGE_DOS_HEADER __ImageBase;
#define PIB_HINSTANCE ((HINSTANCE)&__ImageBase)
#endif // ]

//#define YAPIB_LIST
//#define YAPIB_NODE_FORM
#define PIB_CONTROL_ARRAY2

///////////////////////////////////////////////////////////////////////////////
#define _PIB_STRA(x)  #x
#define PIB_STRA(x)   _PIB_STRA(x)

#define _PIB_STRW(x)  L ## #x
#define PIB_STRW(x)   _PIB_STRW(x)
#define _PIB_WIDEN(x) L ## x
#define PIB_WIDEN(x)  _PIB_WIDEN(x)

#if ! defined (PIB_VERSION_MAJOR) // [
#define PIB_VERSION_MAJOR 0
#endif // ]
#if ! defined (PIB_VERSION_MINOR) // [
#define PIB_VERSION_MINOR 0
#endif // ]
#if ! defined (PIB_VERSION_BUILD) // [
#define PIB_VERSION_BUILD 0

#define PIB_VERSIONA \
    PIB_STRA(PIB_VERSION_MAJOR.PIB_VERSION_MINOR.PIB_VERSION_BUILD)
#endif // ]

#define PIB_LABELA    "[Y]et [A]nother [P]lug-[i]n [B]undle (YAPiB) v"
#define PIB_TITLEA    PIB_LABELA PIB_VERSIONA
#define PIB_TITLEW    PIB_WIDEN(PIB_LABELA) PIB_WIDEN(PIB_VERSIONA)

#define PIB_WS2CS(ws,cs) \
  pib_ws2cs(ws,cs,sizeof cs)

///////////////////////////////////////////////////////////////////////////////
#if defined (__GNUC__) // [
#define PIB_PRIsa L"s"
#define PIB_PRIsw L"S"
#else // ] [
#define PIB_PRIsa L"S"
#define PIB_PRIsw L"s"
#endif // ]

///////////////////////////////////////////////////////////////////////////////
#if ! defined (UNUSED) // [
#ifdef __GNUC__ // [
#define UNUSED __attribute__((__unused__))
#else // ] [
#define UNUSED
#endif // ]
#endif // ]
#if defined (_MSC_VER) && ! defined (__func__) // [
#define __func__ __FUNCTION__
#endif // ]

#if 0 // [
#define DMESSAGE(message) \
  pib_vmessagea("Error " message ". (%s:%d:%s)", \
      pib_basenamea(__FILE__),__LINE__,__func__)
#define DMESSAGEW(message) \
  pib_vmessagew(L"Error " L ## message\
      L". (%" PIB_PRIsa L":%d:%" PIB_PRIsa L")",\
      pib_basenamea(__FILE__),__LINE__,__func__)
#else // ] [
#define DMESSAGE(message) \
  pib_messagea(pib_basenamea(__FILE__),__LINE__,__func__,\
      "Error " message)
#define DMESSAGEW(message) \
  pib_messagew(pib_basenamea(__FILE__),__LINE__,__func__,\
      L"Error " L ## message)
#endif // ]
#if 0 // [
#define DVMESSAGE(message,...) \
  pib_vmessagea("Error " message ". (%s:%d:%s)",__VA_ARGS__, \
      pib_basenamea(__FILE__),__LINE__,__func__)
#define DVMESSAGEW(message,...) \
  pib_vmessagew(L"Error " L ## message\
      L". (%" PIB_PRIsa L":%d:%" PIB_PRIsa L")",\
      __VA_ARGS__,pib_basenamea(__FILE__),__LINE__,__func__)
#else // ] [
#define DVMESSAGE(message,...) \
  pib_vmessagea(pib_basenamea(__FILE__),__LINE__,__func__,\
  "Error " message,__VA_ARGS__)
#define DVMESSAGEW(message,...) \
  pib_vmessagew(pib_basenamea(__FILE__),__LINE__,__func__,\
  L"Error " L ## message,__VA_ARGS__)
#endif // ]

#if defined (DEBUG) // [
#define DCREATE() \
  pib_trace_create()
#define DDESTROY() \
  pib_trace_destroy()
#define DMARKLN() \
  pib_trace_markln(__FILE__,__LINE__,__func__)
#define DWRITELN(format) \
  pib_trace_writeln(__FILE__,__LINE__,__func__,format)
#define DWRITELNW(format) \
  pib_trace_writelnw(__FILE__,__LINE__,__func__,L ## format)
#define DVWRITELN(format,...) \
  pib_trace_vwriteln(__FILE__,__LINE__,__func__,format,__VA_ARGS__)
#define DVWRITELNW(format,...) \
  pib_trace_vwritelnw(__FILE__,__LINE__,__func__,L ## format,__VA_ARGS__)
#else // ] [
#define DCREATE()
#define DDESTROY()
#define DMARKLN()
#define DWRITELN(format)
#define DWRITELNW(format)
#define DVWRITELN(format,...)
#define DVWRITELNW(format,...)
#endif // ]

///////////////////////////////////////////////////////////////////////////////
typedef enum _PibMsg PibMsg;
#if defined (YAPIB_LIST) // [
//typedef struct _YapibNodeVmt YapibNodeVmt;
typedef struct _YapibNode YapibNode;
#if 1 // [
typedef struct _PibCreateParam PibCreateParam;
#endif // ]
#endif // ]
typedef const struct _PibPropertyVmt PibPropertyVmt;
typedef struct _PibProperty PibProperty;
typedef struct _PibProperties PibProperties;
#if defined (PIB_CONTROL_DISABLED) // [
typedef const struct _PibControlVmt PibControlVmt;
typedef const struct _PibRadioGroupPair PibRadioGroupPair;
typedef struct _PibTabConfig PibTabConfig;
typedef /*const*/ struct _PibFormVmt PibFormVmt;
typedef struct _PibForm PibForm;
typedef struct _PibFormInit PibFormInit;
typedef struct _PibTabPage PibTabPage;
typedef struct _PibRadioGroup PibRadioGroup;
typedef struct _PibTab PibTab;
typedef struct _PibCheckBox PibCheckBox;
typedef struct _PibProgressBar PibProgressBar;
typedef struct _PibSlider PibSlider;
typedef struct _PibLabel PibLabel;
typedef struct _PibSysLink PibSysLink;
typedef struct _PibPicture PibPicture;
typedef struct _PibControl PibControl;
typedef const struct _PibControlConfig PibControlConfig;
#endif // ]
typedef const struct _PibThreadRequestVmt PibThreadRequestVmt;
typedef struct _PibThreadRequest PibThreadRequest;
typedef struct _PibThread PibThread;
#if defined (PIB_TIMER) // [
typedef struct _PibTimer PibTimer;
#endif // ]

typedef const struct _PibControlParam2 PibControlParam2;
typedef enum _PibControlType2 PibControlType2;
typedef struct _PibControlVmt2 PibControlVmt2;
typedef struct _PibControl2 PibControl2;
typedef struct _PibModelVmt2 PibModelVmt2;
#if defined (PIB_CONTROL_ARRAY2) // [
typedef struct _PibControlArray2 PibControlArray2;
#endif // ]
typedef struct _PibModel2 PibModel2;

typedef void (*PibControlCallback2)(PibControl2 *control);
typedef void (*PibControlCallbackPaint2)(PibControl2 *control, HDC hdc);
typedef void (*PibControlCallbackParentRect2)(PibControl2 *control,
    RECT *rcParent);

#if defined (YAPIB_LIST) // [
typedef HWND (*YapibDialogCreate)(HWND hWndParent, PibCreateParam *param);
#endif // ]

typedef void pib_property_callback_t(const PibProperty *property,
    const PibProperties *properties, void *data);

#if defined (PIB_CONTROL_DISABLED) // [
typedef void pib_control_callback_t(PibControl *control, HWND hDlg);
typedef void pib_control_paint_t(PibControl *control, HWND hWnd);
#endif // ]

enum _PibMsg {
#if ! defined (YAPIB_LIST) // [
  WM_PIB_REGISTER=WM_APP+100,
#else // ] [
  WM_PIB_PROPERTIES=WM_APP,
  WM_YAPIB_PRESENT,
  WM_YAPIB_REGISTER_REQUEST,
  WM_YAPIB_REGISTER_BROADCAST,
#endif // ]
};

#if defined (YAPIB_LIST) // [
#if 0 // [
struct _YapibNodeVmt {
  void rollback(YapibNode *node);
  void commit(YapibNode *node);
};
#endif // ]

#if 1 // [
struct _YapibNode {
  YapibNode *prev,*next;
#if defined (YAPIB_NODE_FORM) // [
  PibForm *form;
#else // ] [
  PibFormInit *init;
  HWND hDlg;
#endif // ]
};
#endif // ]

struct _PibCreateParam {
  PibFormInit *init;
  YapibDialogCreate dialog_create;
  HINSTANCE hInstance;
  int idd;
};
#endif // ]

///////////////////////////////////////////////////////////////////////////////
struct _PibPropertyVmt {
#if defined (DEBUG) // [
  const char *id;
#endif // ]
  void (*destroy)(PibProperty *property);
  void (*read)(const PibProperty *property, PibProperties *properties);
  void (*write)(const PibProperty *property, PibProperties *properties);
  void (*reset)(const PibProperty *property, PibProperties *properties);
  int (*get_int)(PibProperty *property, void *data);
  void (*set_int)(PibProperty *property, void *data, int i);
  long (*get_long)(PibProperty *property, void *data);
  void (*set_long)(PibProperty *property, void *data, long l);
  double (*get_double)(PibProperty *property, void *data);
  void (*set_double)(PibProperty *property, void *data, double x);
};

#define pib_property_destroy(property) \
  ((property)->vmt->destroy(property))
#define pib_property_id(property) \
  (pib_basenamea((property)->vmt->id))
#define pib_property_read(property,properties) \
  ((property)->vmt->read(property,properties))
#define pib_property_write(property,properties) \
  ((property)->vmt->write(property,properties))
#define pib_property_reset(property,properties) \
  ((property)->vmt->reset(property,properties))
#define pib_property_get_int(property,data) \
  ((property)->vmt->get_int(property,data))
#define pib_property_set_int(property,data,i) \
  ((property)->vmt->set_int(property,data,i))
#define pib_property_get_long(property,data) \
  ((property)->vmt->get_long(property,data))
#define pib_property_set_long(property,data,l) \
  ((property)->vmt->set_long(property,data,l))
#define pib_property_get_double(property,data) \
  ((property)->vmt->get_double(property,data))
#define pib_property_set_double(property,data,x) \
  ((property)->vmt->set_double(property,data,x))

struct _PibProperty {
  PibPropertyVmt *vmt;
  const wchar_t *app;
  const wchar_t *key;
  int offs;

  struct {
    pib_property_callback_t *changed;
    void *data;
  } cb;
};

int pib_property_common_create(PibProperty *property, PibPropertyVmt *vmt,
    const wchar_t *app, const wchar_t *key, int offs,
    pib_property_callback_t *changed, void *data);
void pib_property_common_destroy(PibProperty *property);

void pib_property_fire_changed(const PibProperty *property,
    const PibProperties *properties);

int pib_property_null_create_ex(PibProperty *property, const wchar_t *app,
    const wchar_t *key, int offs,
    pib_property_callback_t *changed, void *data);
int pib_property_int_create_ex(PibProperty *property, const wchar_t *app,
    const wchar_t *key, int offs,
    pib_property_callback_t *changed, void *data);
int pib_property_long_create_ex(PibProperty *property, const wchar_t *app,
    const wchar_t *key, int offs,
    pib_property_callback_t *changed, void *data);
int pib_property_double_create_ex(PibProperty *property, const wchar_t *app,
    const wchar_t *key, int offs,
    pib_property_callback_t *changed, void *data);

#define pib_property_null_create(property,app,key,offs) \
  pib_property_null_create_ex(property,app,key,offs,NULL,NULL)
#define pib_property_int_create(property,app,key,offs) \
  pib_property_int_create_ex(property,app,key,offs,NULL,NULL)
#define pib_property_long_create(property,app,key,offs) \
  pib_property_long_create_ex(property,app,key,offs,NULL,NULL)
#define pib_property_double_create(property,app,key,offs) \
  pib_property_double_create_ex(property,app,key,offs,NULL,NULL)

///////////////////////////////////////////////////////////////////////////////
struct _PibProperties {
  void *cons,*trans;
  PibProperty *min,*max;
  const wchar_t *path;
};

int pib_properties_create(PibProperties *properties, PibProperty *min,
    PibProperty *max, void *cons, void *trans, const wchar_t *path);
void pib_properties_destroy(PibProperties *properties);

void pib_properties_read(PibProperties *properties);
void pib_properties_write(PibProperties *properties);
void pib_properties_reset(PibProperties *properties);

#if defined (PIB_CONTROL_DISABLED) // [
///////////////////////////////////////////////////////////////////////////////
struct _PibFormVmt {
#if defined (DEBUG) // [
  const char *id;
#endif // ]
  /*
   * NOTE:  avoid "const" qualifier because it needs to be assigned
   *   to TCITEMW::pszText which ist non "const"!!!
   */
  /*const*/ wchar_t *label;
  int idd;
  size_t ncontrols;
  PibControlConfig *config;
  void (*destroy)(PibForm *form);
  DLGPROC dlgproc;
};

#define pib_form_destroy(form) \
  ((form)->vmt->destroy(form))
#define pib_form_id(form) \
  (pib_basenamea((form)->vmt->id))
#define pib_form_label(form) \
  ((form)->vmt->label)
#define pib_form_idd(form) \
  ((form)->vmt->idd)
#define pib_form_ncontrols(form) \
  ((form)->vmt->ncontrols)
#define pib_form_config(form) \
  ((form)->vmt->config)
#define pib_form_dlgproc(form) \
  ((form)->vmt->dlgproc)

struct _PibForm {
  PibFormVmt *vmt;
  PibProperties *properties;
  void *data;
  PibControl **min;
  PibControl **max;
};

struct _PibFormInit {
  PibFormVmt *form_vmt;
  PibForm *form;
  PibProperties *properties;
  void *data;
};

int pib_form_common_create(PibFormInit *init, HWND hDlg);
void pib_form_common_destroy(PibForm *form);

void pib_form_post(PibForm *form, HWND hDlg, UINT uMsg, WPARAM wParam,
    LPARAM lParam);
int pib_form_refresh(PibForm *form, HWND hDlg);
/*
 * intended to be called from some event handler when the "idc" is already
 * known and the accociated "val" has to be set in "options".
 */
int pib_form_get(PibForm *form, HWND hDlg, int idc);

///////////////////////////////////////////////////////////////////////////////
struct _PibControlVmt {
#if defined (DEBUG) // [
  const char *id;
#endif // ]
  void (*destroy)(PibControl *control);
  int (*idc_min)(const PibControl *control);
  int (*idc_max)(const PibControl *control);
  int (*get)(PibControl *control, HWND hDlg, int idc);
  void (*notify)(PibControl *control, HWND hDlg, UINT uMsg, WPARAM wParam,
      LPARAM lParam);
  void (*post)(PibControl *control, HWND hDlg, UINT uMsg, WPARAM wParam,
      LPARAM lParam);
};

#define pib_control_id(control) \
  (pib_basenamea((control)->common.vmt->id))
#define pib_control_destroy(control) \
  ((control)->common.vmt->destroy(control))
#define pib_control_idc_min(control) \
  ((control)->common.vmt->idc_min(control))
#define pib_control_idc_max(control) \
  ((control)->common.vmt->idc_max(control))
#define pib_control_get(control,hDlg,idc) \
  ((control)->common.vmt->get(control,hDlg,idc))
#define pib_control_notify(control,hDlg,uMsg,wParam,lParam) \
  ((control)->common.vmt->notify(control,hDlg,uMsg,wParam,lParam))
#define pib_control_post(control,uMsg,wParam,lParam,hDlg) \
  ((control)->common.vmt->post(control,hDlg,uMsg,wParam,lParam))
#define pib_control_refresh(control,hDlg) \
  ((control)->common.vmt->post(control,hDlg,WM_PIB_IPC,PIB_REFRESH,0))

struct _PibRadioGroupPair {
#if defined (DEBUG) // [
  const char *id;
#endif // ]
  int idc;
  int val;
};

struct _PibTabConfig {
  int idc;
  size_t npages;
  PibFormVmt **form_vmts;
};

struct _PibTabPage {
  PibForm form;
  HWND hPage;
};

struct _PibCheckBox {
  int idc;
};

struct _PibRadioGroup {
  size_t nitems;
  PibRadioGroupPair *min;
};

struct _PibTab {
  PibTabConfig config;
  int n;
  PibTabPage pages[1];
};

struct _PibProgressBar {
  int idc;
  int max_offs;
  int state;
};

struct _PibSlider {
  int idc;
  int max_offs;
#if 0 // [
  int state;
#endif // ]
};

struct _PibLabel {
  int idc;
  wchar_t *text;
};

struct _PibSysLink {
  int idc;
  wchar_t *text;
};

struct _PibPicture {
  int idc;
  pib_control_paint_t *paint;
};

struct _PibControl {
  struct {
    PibControlVmt *vmt;
    PibProperty *property;
    PibForm *form;

    struct {
      pib_control_callback_t *changed;
    } cb;
  } common;

  union {
    PibCheckBox check_box;
    PibRadioGroup radio_group;
    PibTab tab;
    PibProgressBar progress_bar;
    PibSlider slider;
    PibLabel label;
    PibSysLink sys_link;

    struct {
      PibPicture config;
      HWND hWnd;
    } picture;
  };
};

#define pib_control_form(control) \
  ((control)->common.form)
#define pib_control_cons(control) \
  (pib_control_form(control)->properties->cons)
#define pib_control_trans(control) \
  (pib_control_form(control)->properties->trans)

PibControl *pib_control_common_new(size_t size, PibControlVmt *vmt,
    PibControlConfig *config, PibForm *form);
void pib_control_common_destroy(PibControl *control);

void pib_control_changed(PibControl *control, HWND hDlg);

///////////////////////////////////////////////////////////////////////////////
struct _PibControlConfig {
  PibControl *(*control_new)(PibControlConfig *config, PibForm *form,
      HWND hDlg);

  int property_offs;
  pib_control_callback_t *changed;

  union {
    PibCheckBox check_box;
    PibRadioGroup radio_group;
    PibTabConfig tab;
    PibProgressBar progress_bar;
    PibSlider slider;
    PibLabel label;
    PibSysLink sys_link;
    PibPicture picture;
  };
};

PibControl *pib_control_check_box_new(PibControlConfig *config,
    PibForm *form, HWND hDlg);
PibControl *pib_control_radio_group_new(PibControlConfig *config,
    PibForm *form, HWND hDlg);
PibControl *pib_control_tab_new(PibControlConfig *config,
    PibForm *form, HWND hDlg);
PibControl *pib_control_progress_bar_new(PibControlConfig *config,
    PibForm *form, HWND hDlg);
PibControl *pib_control_slider_new(PibControlConfig *config,
    PibForm *form, HWND hDlg);
PibControl *pib_control_label_new(PibControlConfig *config,
    PibForm *form, HWND hDlg);
PibControl *pib_control_sys_link_new(PibControlConfig *config,
    PibForm *form, HWND hDlg);
PibControl *pib_control_picture_new(PibControlConfig *config,
    PibForm *form, HWND hDlg);
#endif // ]

///////////////////////////////////////////////////////////////////////////////
struct _PibThreadRequestVmt {
  const char *id;
  void (*destroy)(PibThreadRequest *request);
  void (*execute)(PibThreadRequest *request);
};

#define pib_thread_request_id(request) \
  (pib_basenamea((request)->vmt->id))
#define pib_thread_request_destroy(request) \
  ((request)->vmt->destroy(request))
#define pib_thread_request_execute(request) \
  ((request)->vmt->execute(request))

struct _PibThreadRequest {
  PibThreadRequestVmt *vmt;
  PibThread *thread;
  PibThreadRequest *prev;
  PibThreadRequest *next;
};

void *pib_thread_request_common_new(PibThreadRequestVmt *vmt,
    PibThread *thread, size_t size);
void pib_thread_request_common_destroy(PibThreadRequest *request);

PibThreadRequest *pib_thread_request_stop_new(PibThread *thread);

struct _PibThread {
  struct {
    int next;
  } request;

  void *data;
  PibThreadRequest *head;
  PibThreadRequest *tail;
  HANDLE hMutex;
  HANDLE hEvent;
  HANDLE hThread;
};

int pib_thread_common_create(PibThread *thread, void *data);
void pib_thread_common_destroy(PibThread *thread);

void pib_thread_send(PibThread *thread, PibThreadRequest *request);
PibThreadRequest *pib_thread_receive(PibThread *thread);

#if defined (PIB_TIMER) // [
///////////////////////////////////////////////////////////////////////////////
struct _PibTimer {
  int nouse;
};
#endif // ]

///////////////////////////////////////////////////////////////////////////////
struct _PibControlParam2 {
  PibControl2 *(*create)(HWND hDlg, PibControl2 *parent,
      PibControlParam2 *param, PibModel2 *model);
  int idc;

  union {
    struct {
      const wchar_t *label;
    } link;

    struct {
      int index;
    } tabcontrol;

    struct {
      struct {
        int x,y;
      } index;
    } form;

    struct {
      PibControlCallback2 cb;
    } label;

    struct {
      PibControlCallback2 cb;
    } progress;

    struct {
      PibControlCallbackPaint2 cb;
    } image;

    struct {
      PibControlCallback2 cb;
    } button;

    struct {
      int index;
      PibControlCallback2 cb;
    } check;

    struct {
      int value;
    } radio;

    struct {
      int index;
    } radiogroup;

    struct {
      PibControlCallbackParentRect2 cb;
    } proxy;
  };

  struct {
    int nchildren;
    PibControlParam2 *const *children;
#if defined (PIB_IMAGE_BASE) // [
    HINSTANCE hInstance;
#else // ] [
    const wchar_t *module;
#endif // ]
    int idd;

    union {
      const wchar_t *title;
      wchar_t *label;
    };

    DLGPROC DlgProc;
  } branch;
};

PibControl2 *pib_control2_root(PibControl2 *control);
PibProperty *pib_control2_property(PibControl2 *control, int index);

////////
enum _PibControlType2 {
  PIB_CONTROL_TYPE_NULL,
  PIB_CONTROL_TYPE_LABEL,
  PIB_CONTROL_TYPE_IMAGE,
  PIB_CONTROL_TYPE_CHECK,
  PIB_CONTROL_TYPE_TREE,
  PIB_CONTROL_TYPE_RADIO,
  PIB_CONTROL_TYPE_PROGRESS,
  PIB_CONTROL_TYPE_LINK,
  PIB_CONTROL_TYPE_BUTTON,
  PIB_CONTROL_TYPE_FORM,
  PIB_CONTROL_TYPE_TABCONTROL,
  PIB_CONTROL_TYPE_PROXY,
  PIB_CONTROL_TYPE_RADIOGROUP,
  PIB_CONTROL_TYPE_TABPAGE,
  PIB_CONTROL_TYPE_MAX,
};

#if 0 && defined (DEBUG) // [
extern const char *pib_mnemonic_control_type2[PIB_CONTROL_TYPE_MAX];
#endif // ]

////////
struct _PibControlVmt2 {
  const char *id;
  PibControlType2 type;
  void (*destroy)(PibControl2 *control);

  PibControl2 *(*create_ex)(PibControl2 *control, HWND hDlg);
  void (*adjust)(PibControl2 *control, PibControl2 *parent);
  void (*refresh)(PibControl2 *control);
  void (*init)(PibControl2 *control);
  void (*close)(PibControl2 *control);
  void (*notify)(PibControl2 *control, WPARAM wParam, LPARAM lParam);
  void (*command)(PibControl2 *control, WPARAM wParam, LPARAM lParam);
};

////////
struct _PibControl2 {
  PibControlVmt2 *vmt;
  PibControl2 *parent;
  PibControlParam2 *param;
  PibModel2 *model;
  PibProperties *properties;

  union {
    HWND hWnd;
    HWND hDlg;
  };

  union {
    struct {
      struct {
        PibProperty *cur,*max;
      } property;
    } label;

    struct {
      struct {
        PibProperty *npage;
      } property;
    } tabcontrol;

    struct {
      struct {
        PibProperty *x,*y;
      } property;
    } form;

    struct {
      struct {
        PibProperty *checked;
      } property;
    } check;

    struct {
      struct {
        PibProperty *value;
      } property;
    } radiogroup;
  };

  struct {
    union {
      struct {
        int npage;
        int nchildren;
      } tabcontrol;
    };

    int nchildren;
    PibControl2 *children[1];
  } branch;
};

PibControl2 *pib_leaf_create(HWND hDlg, PibControl2 *parent,
    PibControlParam2 *param, PibModel2 *model, PibControlVmt2 *vmt);
void _pib_leaf_destroy(PibControl2 *control);

PibControl2 *pib_branch_create(HWND hDlg, PibControl2 *parent, int dialog,
    PibControlParam2 *param, PibModel2 *model, PibControlVmt2 *vmt);
void _pib_branch_destroy(PibControl2 *control);

void _pib_branch_rect_center(const RECT *rcParent, const RECT *rcChild,
    RECT *rcResult);
void _pib_branch_refresh(PibControl2 *control);

PibControl2 *pib_button_create(HWND hDlg, PibControl2 *parent,
    PibControlParam2 *param, PibModel2 *model);
PibControl2 *pib_link_create(HWND hDlg,  PibControl2 *parent,
    PibControlParam2 *param, PibModel2 *model);
PibControl2 *pib_tree_create(HWND hDlg,  PibControl2 *parent,
    PibControlParam2 *param, PibModel2 *model);
PibControl2 *pib_check_create(HWND hDlg,  PibControl2 *parent,
    PibControlParam2 *param, PibModel2 *model);
PibControl2 *pib_radio_create(HWND hDlg,  PibControl2 *parent,
    PibControlParam2 *param, PibModel2 *model);
PibControl2 *pib_progress_create(HWND hDlg,  PibControl2 *parent,
    PibControlParam2 *param, PibModel2 *model);
PibControl2 *pib_image_create(HWND hDlg,  PibControl2 *parent,
    PibControlParam2 *param, PibModel2 *model);
PibControl2 *pib_label_create(HWND hDlg,  PibControl2 *parent,
    PibControlParam2 *param, PibModel2 *model);

PibControl2 *pib_radiogroup_create(HWND hDlg,  PibControl2 *parent,
    PibControlParam2 *param, PibModel2 *model);
PibControl2 *pib_tabcontrol_create(HWND hDlg,  PibControl2 *parent,
    PibControlParam2 *param, PibModel2 *model);
PibControl2 *pib_tabpage_create(HWND hDlg, PibControl2 *parent,
    PibControlParam2 *param, PibModel2 *model);
PibControl2 *pib_proxy_create(HWND hDlg,  PibControl2 *parent,
    PibControlParam2 *param, PibModel2 *model);

PibControl2 *pib_form_create(HWND hDlg, PibControlParam2 *param,
    PibModel2 *model);


#define PIB_LINK(IDC,LABEL) \
  { pib_link_create,.idc=IDC, .link={ .label=LABEL } }
#define PIB_PROGRESS(IDC,CALLBACK) \
  { pib_progress_create,.idc=IDC,.progress={ .cb=CALLBACK } }
#define PIB_TREE(IDC) \
  { pib_tree_create, .idc=IDC }
#define PIB_CHECK(IDC,INDEX,CALLBACK) \
  { pib_check_create,.idc=IDC, .check={ .index=1+(INDEX), .cb=CALLBACK } }
#define PIB_RADIO(IDC,VALUE) \
  { pib_radio_create,.idc=IDC, .radio={ .value=VALUE} }
#define PIB_IMAGE(IDC,CALLBACK) \
  { pib_image_create,.idc=IDC,.image={ .cb=CALLBACK } }
#define PIB_LABEL(IDC,CALLBACK) \
  { pib_label_create,.idc=IDC,.label={ .cb=CALLBACK } }
#define PIB_BUTTON(IDC,CALLBACK) \
  { pib_button_create,.idc=IDC,.button={ .cb=CALLBACK } }
#define PIB_FORM(IDD,TITLE,BRANCH,X,Y) \
  { NULL, .form.index={ .x=1+X, .y=1+Y }, \
      .branch={ .idd=IDD, .title=TITLE, \
      .hInstance=PIB_HINSTANCE, \
      .nchildren=(sizeof BRANCH)/(sizeof BRANCH[0]),.children=BRANCH } }
#define PIB_RADIOGROUP(BRANCH,INDEX) \
  { pib_radiogroup_create, .radiogroup={ .index=(INDEX)+1 }, \
      .branch={ .hInstance=PIB_HINSTANCE, \
      .nchildren=(sizeof BRANCH)/(sizeof BRANCH[0]),.children=BRANCH } }
#define PIB_TABCONTROL(IDC,BRANCH,INDEX) \
  { pib_tabcontrol_create, .idc=IDC, .tabcontrol.index=1+(INDEX), \
      .branch={ \
      .nchildren=(sizeof BRANCH)/(sizeof BRANCH[0]),.children=BRANCH } }

#if defined (PIB_PAGE_MORPHED) // [
#define PIB_PAGE(IDD,TITLE,CALLBACK,BRANCH) \
  { pib_page_create, .page={ .cb=CALLBACK }, .branch={ .idd=IDD, \
      .title=TITLE, .hInstance=PIB_HINSTANCE, \
      .nchildren=(sizeof BRANCH)/(sizeof BRANCH[0]),.children=BRANCH } }
#else // ] [
#define PIB_PROXY(IDD,BRANCH,CALLBACK) \
  { pib_proxy_create, .proxy={ .cb=CALLBACK }, \
      .branch={ .idd=IDD, .hInstance=PIB_HINSTANCE, \
      .nchildren=(sizeof BRANCH)/(sizeof BRANCH[0]),.children=BRANCH } }
#define PIB_TABPAGE(IDD,TITLE,BRANCH) \
  { pib_tabpage_create, .branch={ .idd=IDD, \
      .title=TITLE, .hInstance=PIB_HINSTANCE, \
      .nchildren=(sizeof BRANCH)/(sizeof BRANCH[0]),.children=BRANCH } }
#endif // ]

#if defined (PIB_CONTROL_ARRAY2) // [
///////////////////////////////////////////////////////////////////////////////
struct _PibControlArray2 {
  PibControl2 **min,**cur,**max;
};

int pib_control_array_create(PibControlArray2 *array);
void pib_control_array_destroy(PibControlArray2 *array);

int pib_control_array_add(PibControlArray2 *array, PibControl2 *control);
#endif // ]

///////////////////////////////////////////////////////////////////////////////
#define PIB_MODEL_ROOT
struct _PibModelVmt2 {
  PibProperties *(*properties)(PibModel2 *model);
  PibProperty *(*property)(PibModel2 *model, int index);

#if defined (PIB_MODEL_ROOT) // [
  PibControl2 *(*root)(PibModel2 *model);
#endif // ]
  void (*commit)(PibModel2 *model);
  void (*rollback)(PibModel2 *model);
};

struct _PibModel2 {
  PibModelVmt2 *vmt;
};

void pib_model_commit(PibModel2 *model);
void pib_model_rollback(PibModel2 *model);

///////////////////////////////////////////////////////////////////////////////
const wchar_t *pib_basenamew(const wchar_t *path);
const char *pib_basenamea(const char *path);

///////////////////////////////////////////////////////////////////////////////
#if 0 // [
void pib_messagea(const char *messagea);
void pib_vmessagea(const char *message, ...);
#else // ] [
void pib_messagea(const char *file, int line, const char *func,
    const char *message);
void pib_vmessagea(const char *file, int line, const char *func,
    const char *message, ...);
#endif // ]

#if 0 // [
void pib_messagew(const wchar_t *message);
void pib_vmessagew(const wchar_t *message, ...);
#else // ] [
void pib_messagew(const char *file, int line, const char *func,
    const wchar_t *message);
void pib_vmessagew(const char *file, int line, const char *func,
    const wchar_t *message, ...);
#endif // ]

///////////////////////////////////////////////////////////////////////////////
char *pib_ws2cs(const wchar_t *ws, char *cs, size_t size);

#if defined (DEBUG) // [
///////////////////////////////////////////////////////////////////////////////
int pib_trace_create(void);
void pib_trace_destroy(void);

void pib_trace_writeln(const char *path, int line, const char *func,
    const char *format);
void pib_trace_writelnw(const char *path, int line, const char *func,
    const wchar_t *format);
void pib_trace_vwriteln(const char *path, int line, const char *func,
    const char *format, ...);
void pib_trace_vwritelnw(const char *path, int line, const char *func,
    const wchar_t *format, ...);
void pib_trace_markln(const char *path, int line, const char *func);
#endif // ]

///////////////////////////////////////////////////////////////////////////////
int pib_create_ini_path(wchar_t **pathw, const wchar_t *namew);
void pib_destroy_ini_path(wchar_t **pathw);

HWND pib_hwnd_parent(void);

///////////////////////////////////////////////////////////////////////////////
HINSTANCE plugin_hinstance(void);
HWND plugin_hwnd_main(void);
const char *plugin_titlea(void);
const wchar_t *plugin_titlew(void);

//#define PIB_GEN
#if defined (PIB_GEN) // [
///////////////////////////////////////////////////////////////////////////////
__declspec (dllexport)
winampGeneralPurposePlugin *winampGetGeneralPurposePlugin();
#endif // ]

#endif // ]
