/*
 * pib_control2_array.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib.h>

///////////////////////////////////////////////////////////////////////////////
int pib_control_array_create(PibControlArray2 *array)
{
  enum { SIZE=8 };
  array->min=malloc(SIZE*sizeof *array->min);

  if (!array->min) {
    DMESSAGE("allocating");
    goto e_malloc;
  }

  array->cur=array->min;
  array->max=array->min+SIZE;

  return 0;
//cleanup;
  free(array->min);
e_malloc:
  return -1;
}

void pib_control_array_destroy(PibControlArray2 *array)
{
  free(array->min);
}

///////////////////////////////////////////////////////////////////////////////
static int pib_control_array_realloc(PibControlArray2 *array)
{
  size_t size=array->max-array->min;
  size_t offs=array->cur-array->min;
  PibControl2 **min;

  size<<=1;

  if (!size) {
    DMESSAGE("overflow");
    goto e_overflow;
  }

  min=realloc(array->min,size);

  if (!min) {
    DMESSAGE("reallocating");
    goto e_realloc;
  }

  array->min=min;
  array->cur=min+offs;
  array->max=min+size;

  return 0;
//cleanup:
e_realloc:
e_overflow:
  return -1;
}

int pib_control_array_add(PibControlArray2 *array, PibControl2 *control)
{
  if (array->cur==array->max&&pib_control_array_realloc(array)<0) {
    DMESSAGE("reallocating");
    goto e_realloc;
  }

  *array->cur++=control;

  return 0;
e_realloc:
  return -1;
}
