/*
 * pib_control2_check.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib.h>

///////////////////////////////////////////////////////////////////////////////
static PibControlVmt2 vmt;

PibControl2 *pib_check_create(HWND hDlg, PibControl2 *parent,
    PibControlParam2 *param, PibModel2 *model)
{
  return pib_leaf_create(hDlg,parent,param,model,&vmt);
}

static void _pib_check_refresh(PibControl2 *control)
{
  void *trans=control->properties->trans;
  int checked=pib_property_get_int(control->check.property.checked,trans);

  checked=checked?BST_CHECKED:BST_UNCHECKED;

  if (checked==SendMessageW(control->hWnd,BM_GETCHECK,0,0))
    return;

  SendMessageW(control->hWnd,BM_SETCHECK,checked,0);
}

static void _pib_check_adjust(PibControl2 *control, PibControl2 *parent)
{
  /////////////////////////////////////////////////////////////////////////////
  control->check.property.checked
      =pib_control2_property(control,control->param->check.index);

  if (!control->check.property.checked) {
    DVMESSAGE("missing check property (%d)",control->param->check.index);
    return;
  }

  /////////////////////////////////////////////////////////////////////////////
  _pib_check_refresh(control);
}

static void _pib_check_command(PibControl2 *control, WPARAM wParam,
    LPARAM lParam)
{
  int checked=SendMessageW(control->hWnd,BM_GETCHECK,0,0);
  void *trans=control->properties->trans;

  checked=BST_CHECKED==checked?1:0;

  if (checked==pib_property_get_int(control->check.property.checked,trans))
    return;

  pib_property_set_int(control->check.property.checked,trans,checked);

  if (control->param->check.cb)
    control->param->check.cb(control);
}

static PibControlVmt2 vmt={
  .id="CHECK",
  .type=PIB_CONTROL_TYPE_CHECK,
  .destroy=_pib_leaf_destroy,
  .adjust=_pib_check_adjust,
  .refresh=_pib_check_refresh,
  .command=_pib_check_command,
};
