/*
 * pib_control2_form.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib.h>

//#define PIB_FORM_ENABLE_THEME
#if defined (PIB_FORM_ENABLE_THEME) // [
#include <uxtheme.h>
#endif // ]

static PibControlVmt2 vmt;

PibControl2 *pib_form_create(HWND hDlg, PibControlParam2 *param,
    PibModel2 *model)
{
  return pib_branch_create(hDlg,NULL,1,param,model,&vmt);
}

static void _pib_form_close(PibControl2 *control)
{
  PibModel2 *model=control->model;
  void *trans=control->properties->trans;
  RECT rect;

  /////////////////////////////////////////////////////////////////////////////
  model->vmt->rollback(model);

  /////////////////////////////////////////////////////////////////////////////
  GetWindowRect(control->hDlg,&rect);
  pib_property_set_int(control->form.property.x,trans,rect.left);
  pib_property_set_int(control->form.property.y,trans,rect.top);
  pib_properties_write(control->properties);
}

#if defined (PIB_FORM_ENABLE_THEME) // [
static PibControl2 *_pib_form_create_ex(PibControl2 *control, HWND hDlg)
{
  EnableThemeDialogTexture(
    control->hDlg,                // _In_ HWND  hwnd,
    ETDT_ENABLETAB                // _In_ DWORD dwFlags
  );

  return control;
}
#endif // ]

static void _pib_form_adjust(PibControl2 *control, PibControl2 *parent)
{
  enum {
    SWP_FLAGS=SWP_NOZORDER|SWP_NOSIZE|SWP_SHOWWINDOW,
  };

  void *cons=control->properties->cons;
  int x,y;
  UINT uFlags;

  /////////////////////////////////////////////////////////////////////////////
  control->form.property.x
      =pib_control2_property(control,control->param->form.index.x);

  if (!control->form.property.x) {
    DVMESSAGE("missing x property (%d)",control->param->form.index.x);
    return;
  }

  /////////////////////////////////////////////////////////////////////////////
  control->form.property.y
      =pib_control2_property(control,control->param->form.index.y);

  if (!control->form.property.y) {
    DVMESSAGE("missing y property (%d)",control->param->form.index.y);
    return;
  }

  /////////////////////////////////////////////////////////////////////////////
  x=pib_property_get_int(control->form.property.x,cons);
  y=pib_property_get_int(control->form.property.y,cons);
  uFlags=SWP_FLAGS|(x<0||y<0?SWP_NOMOVE:0);

  SetWindowTextW(
    control->hDlg,                // HWND   hWnd,
    control->param->branch.title  // LPCSTR lpString
  );

  SetWindowPos(
    control->hDlg,                // HWND hWnd,
    NULL,                         // HWND hWndInsertAfter,
    x,                            // int  X,
    y,                            // int  Y,
    0,                            // int  cx,
    0,                            // int  cy,
    uFlags                        // UINT uFlags
  );
}

static PibControlVmt2 vmt={
  .id="FORM",
  .type=PIB_CONTROL_TYPE_FORM,
#if defined (PIB_FORM_ENABLE_THEME) // [
  .create_ex=_pib_form_create_ex,
#endif // ]
  .destroy=_pib_branch_destroy,
  //.init=_pib_form_init,
  .close=_pib_form_close,
  .refresh=_pib_branch_refresh,
  .adjust=_pib_form_adjust,
};
