/*
 * pib_control2_image.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib.h>

//#define PIB_IMAGE_VALIDATE

///////////////////////////////////////////////////////////////////////////////
static PibControlVmt2 vmt;
static WNDCLASSW *pib_image_wndclassw(void);

PibControl2 *pib_image_create(HWND hDlg, PibControl2 *parent,
    PibControlParam2 *param, PibModel2 *model)
{
  return pib_leaf_create(hDlg,parent,param,model,&vmt);
}

static PibControl2 *_pib_image_create_ex(PibControl2 *control, HWND hDlg)
{
  WNDCLASSW *wndclassw=pib_image_wndclassw();
  HINSTANCE hInstance=(HINSTANCE)GetWindowLongPtrW(hDlg,GWL_HINSTANCE);
  HFONT hFont=(HFONT)SendMessageW(control->hWnd,WM_GETFONT,0,0);
  RECT rcDlg,rcWnd;

  ////////
  GetWindowRect(hDlg,&rcDlg);
  GetWindowRect(control->hWnd,&rcWnd);
  rcWnd.left-=rcDlg.left;
  rcWnd.right-=rcDlg.left;
  rcWnd.top-=rcDlg.top;
  rcWnd.bottom-=rcDlg.top;
  DestroyWindow(control->hWnd);

  ////////
  control->hWnd=CreateWindowExW(
    0,                                    // DWORD     dwExStyle,
    wndclassw->lpszClassName,             // LPCWSTR   lpClassName,
    NULL,                                 // LPCWSTR   lpWindowName,
    WS_BORDER|WS_VISIBLE|WS_CHILD,        // DWORD     dwStyle,
    rcWnd.left,                           // int       X,
    rcWnd.top,                            // int       Y,
    (rcWnd.right-rcWnd.left),             // int       nWidth,
    (rcWnd.bottom-rcWnd.top),             // int       nHeight,
    hDlg,                                 // HWND      hWndParent,
    (HMENU)control->param->idc,           // HMENU     hMenu,
    hInstance,                            // HINSTANCE hInstance,
    control                               // LPVOID    lpParam
  );

  ////////
  SendMessageW(control->hWnd,WM_SETFONT,(WPARAM)hFont,TRUE);
  SetWindowLongPtrW(control->hWnd,GWLP_USERDATA,(LONG_PTR)control);

  ////////
  return control;
}

static void _pib_image_refresh(PibControl2 *control)
{
  if (control->param->image.cb)
    InvalidateRect(control->hWnd,NULL,TRUE);
}

static void _pib_image_adjust(PibControl2 *control, PibControl2 *parent)
{
  _pib_image_refresh(control);
}

static LRESULT CALLBACK pib_image_window_procw(HWND hWnd, UINT uMsg,
    WPARAM wParam, LPARAM lParam)
{
  PibControl2 *control=(PibControl2 *)GetWindowLongPtrW(hWnd,GWLP_USERDATA);
  PAINTSTRUCT ps;
  HDC hdc;

  switch (uMsg) {
  case WM_CREATE:
    control=((CREATESTRUCTW *)lParam)->lpCreateParams;

    SetWindowLongPtrW(
      hWnd,                         // HWND     hWnd,
      GWLP_USERDATA,                // int      nIndex,
      (LONG_PTR)control             // LONG_PTR dwNewLong
    );

    break;
  case WM_PAINT:
    // WM_PAINT message
    //   https://docs.microsoft.com/en-us/windows/win32/gdi/wm-paint
    hdc=BeginPaint(control->hWnd,&ps); // {

    // All painting occurs here, between BeginPaint and EndPaint.
    if (control->param->image.cb)
      control->param->image.cb(control,hdc);
    else
      FillRect(hdc,&ps.rcPaint,(HBRUSH)(COLOR_WINDOW+1));

    EndPaint(control->hWnd,&ps); // }
#if defined (PIB_IMAGE_VALIDATE) // [
    ValidateRect(control->hWnd,NULL);
#endif // ]

    return 0;
  default:
    break;
  }

  return DefWindowProcW(
    hWnd,                           // HWND   hWnd,
    uMsg,                           // UINT   Msg,
    wParam,                         // WPARAM wParam,
    lParam                          // LPARAM lParam
  );
}

static WNDCLASSW *pib_image_wndclassw(void)
{
  static WNDCLASSW wndclassw;

  if (!wndclassw.lpszClassName) {
#if 0 // [
    wndclassw.style=0;              // UINT      style;
#endif // ]
    wndclassw.lpfnWndProc=pib_image_window_procw;
                                    // WNDPROC   lpfnWndProc;
#if 0 // [
    wndclassw.cbClsExtra=0;         // int       cbClsExtra;
    wndclassw.cbWndExtra=0;         // int       cbWndExtra;
#endif // ]
    wndclassw.hInstance=plugin_hinstance();
                                    // HINSTANCE hInstance;
#if 0 // [
    wndclassw.hIcon=NULL;           // HICON     hIcon;
    wndclassw.hCursor=NULL;         // HCURSOR   hCursor;
#endif // ]
#if 0 // [
    wndclassw.hbrBackground=(HBRUSH)GetStockObject(LTGRAY_BRUSH);
                                    // HBRUSH    hbrBackground;
#else // ] [
    wndclassw.hbrBackground=(HBRUSH)(COLOR_BTNFACE+1),
                                    // HBRUSH    hbrBackground;
#endif // ]
#if 0 // [
    wndclassw.lpszMenuName=NULL;    // LPCWSTR   lpszMenuName;
#endif // ]
    wndclassw.lpszClassName=L"PIB_IMAGE_CONTROL";
                                    // LPCWSTR   lpszClassName;
    RegisterClassW(&wndclassw);
  }

  return &wndclassw;
}

static PibControlVmt2 vmt={
  .id="IMAGE",
  .type=PIB_CONTROL_TYPE_IMAGE,
  .create_ex=_pib_image_create_ex,
  .adjust=_pib_image_adjust,
  .refresh=_pib_image_refresh,
  .destroy=_pib_leaf_destroy,
};
