/*
 * pib_control2_link.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib.h>

///////////////////////////////////////////////////////////////////////////////
PibControl2 *pib_leaf_create(HWND hDlg, PibControl2 *parent,
    PibControlParam2 *param, PibModel2 *model, PibControlVmt2 *vmt)
{
  size_t size=(sizeof (PibControl2))-(sizeof (((PibControl2 *)NULL)->branch));
  PibControl2 *control=calloc(1,size);

DVWRITELN("=== \"%s\"",vmt->id);
  ////////
  if (!control) {
    DMESSAGE("allocating control");
    goto e_control;
  }

  ////////
  control->vmt=vmt;
  control->param=param;
  control->model=model;
  control->parent=parent;

  ////////
  control->properties=model->vmt->properties(model);
  
  if (!control->properties) {
    DMESSAGE("missing properties");
    goto e_properties;
  }

  ////////
  control->hWnd=GetDlgItem(hDlg,param->idc);

//DVWRITELN("1 %p: %p (%p)",hDlg,control->hWnd,parent?parent->hDlg:NULL);
  if (!control->hWnd&&parent&&parent->parent) {
    control->hWnd=GetDlgItem(parent->parent->hDlg,param->idc);
//DVWRITELN("2 2p: %p",parent->parent->hDlg,control->hWnd);
  }

  if (!control->hWnd) {
    if (parent) {
      DVMESSAGE("control (idc:%d) not found in dialog (idd:%d)",
          param->idc,parent->param->branch.idd);
    }
    else
      DVMESSAGE("control (idc:%d) not found",param->idc);

    goto e_notfound;
  }

  SetWindowLongPtrW(control->hWnd,GWLP_USERDATA,(LONG_PTR)control);

  if (vmt->create_ex)
    vmt->create_ex(control,hDlg);

  if (vmt->adjust)
    vmt->adjust(control,parent);

  return control;
//cleanup:
e_notfound:
e_properties:
  free(control);
e_control:
  return NULL;
}

void _pib_leaf_destroy(PibControl2 *control)
{
  free(control);
}

#if 0 // [
void _pib_leaf_postfix(PibControl2 *control, UINT uMsg, WPARAM wParam,
   LPARAM lParam)
{
  SendMessage(control->hWnd,uMsg,wParam,lParam);
}
#endif // ]
