/*
 * pib_control2_link.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib.h>

///////////////////////////////////////////////////////////////////////////////
static PibControlVmt2 vmt;

PibControl2 *pib_link_create(HWND hDlg, PibControl2 *parent,
    PibControlParam2 *param, PibModel2 *model)
{
  return pib_leaf_create(hDlg,parent,param,model,&vmt);
}

static PibControl2 *_pib_link_create_ex(PibControl2 *control, HWND hDlg)
{
  HINSTANCE hInstance=(HINSTANCE)GetWindowLongPtrW(hDlg,GWL_HINSTANCE);
  HFONT hFont=(HFONT)SendMessageW(control->hWnd,WM_GETFONT,0,0);
  RECT rcDlg,rcWnd;

  ////////
  GetWindowRect(hDlg,&rcDlg);
  GetWindowRect(control->hWnd,&rcWnd);
  rcWnd.left-=rcDlg.left;
  rcWnd.right-=rcDlg.left;
  rcWnd.top-=rcDlg.top;
  rcWnd.bottom-=rcDlg.top;
  DestroyWindow(control->hWnd);

  ////////
  control->hWnd=CreateWindowExW(
    0,                                    // DWORD     dwExStyle,
    WC_LINK,                              // LPCWSTR   lpClassName,
    control->param->link.label,           // LPCWSTR   lpWindowName,
    WS_VISIBLE|WS_CHILD|WS_TABSTOP,       // DWORD     dwStyle,
    rcWnd.left,                           // int       X,
    rcWnd.top,                            // int       Y,
    (rcWnd.right-rcWnd.left),             // int       nWidth,
    (rcWnd.bottom-rcWnd.top),             // int       nHeight,
    hDlg,                                 // HWND      hWndParent,
    (HMENU)control->param->idc,           // HMENU     hMenu,
    hInstance,                            // HINSTANCE hInstance,
    NULL                                  // LPVOID    lpParam
  );

  ////////
  SendMessageW(control->hWnd,WM_SETFONT,(WPARAM)hFont,TRUE);
  SetWindowLongPtrW(control->hWnd,GWLP_USERDATA,(LONG_PTR)control);

  ////////
  return control;
}

static void _pib_link_notify(PibControl2 *control, WPARAM wParam,
    LPARAM lParam)
{
  if (NM_CLICK==((LPNMHDR)lParam)->code) {
DVWRITELNW("\"%" PIB_PRIsw L"\"",((PNMLINK)lParam)->item.szUrl);
    ShellExecuteW(
      NULL,                               // HWND    hwnd,
      L"open",                            // LPCWSTR lpOperation,
      ((PNMLINK)lParam)->item.szUrl,     // LPCWSTR lpFile,
      NULL,                               // LPCWSTR lpParameters,
      NULL,                               // LPCWSTR lpDirectory,
      SW_SHOW                             // INT     nShowCmd
    );
  }
}

static PibControlVmt2 vmt={
  .id="LINK",
  .type=PIB_CONTROL_TYPE_LINK,
  .create_ex=_pib_link_create_ex,
  .destroy=_pib_leaf_destroy,
  .notify=_pib_link_notify,
};
