/*
 * pib_control2_radiogroup.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib.h>

static PibControlVmt2 vmt;

PibControl2 *pib_radiogroup_create(HWND hDlg, PibControl2 *parent,
    PibControlParam2 *param, PibModel2 *model)
{
  return pib_branch_create(hDlg,parent,0,param,model,&vmt);
}

static void _pib_radiogroup_refresh(PibControl2 *control)
{
  void *trans=control->properties->trans;
  int value=pib_property_get_int(control->radiogroup.property.value,trans);
  int nchildren=control->param->branch.nchildren;
  PibControl2 **cur=control->branch.children;
  PibControl2 **max=cur+nchildren;

  while (cur<max) {
    if ((*cur)->param->radio.value==value) {
      if (BST_UNCHECKED==SendMessageW((*cur)->hWnd,BM_GETCHECK,0,0))
        SendMessageW((*cur)->hWnd,BM_SETCHECK,BST_CHECKED,0);

      pib_property_set_int(control->radiogroup.property.value,trans,
          (*cur)->param->radio.value);
    }
    else if (BST_CHECKED==SendMessageW((*cur)->hWnd,BM_GETCHECK,0,0))
      SendMessageW((*cur)->hWnd,BM_SETCHECK,BST_UNCHECKED,0);

    ++cur;
  }
}

static void _pib_radiogroup_adjust(PibControl2 *control, PibControl2 *parent)
{
  /////////////////////////////////////////////////////////////////////////////
  control->radiogroup.property.value
      =pib_control2_property(control,control->param->radiogroup.index);

  if (!control->radiogroup.property.value) {
    DVMESSAGE("missing radiogroup property (%d)",
        control->param->radiogroup.index);
    return;
  }

  /////////////////////////////////////////////////////////////////////////////
  _pib_radiogroup_refresh(control);
}

static PibControlVmt2 vmt={
  .id="RADIOGROUP",
  .type=PIB_CONTROL_TYPE_RADIOGROUP,
  .destroy=_pib_branch_destroy,
  .adjust=_pib_radiogroup_adjust,
  .refresh=_pib_radiogroup_refresh,
};
