/*
 * pib_control2_tabcontrol.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib.h>

//#define PIB_TABCONTROL_ENABLE_THEME
#if defined (PIB_TABCONTROL_ENABLE_THEME) // [
#include <uxtheme.h>
#endif // ]

///////////////////////////////////////////////////////////////////////////////
static PibControlVmt2 vmt;

static void _pib_tabcontrol_set(PibControl2 *control, int n);

PibControl2 *pib_tabcontrol_create(HWND hDlg, PibControl2 *parent,
    PibControlParam2 *param, PibModel2 *model)
{
  return pib_branch_create(hDlg,parent,1,param,model,&vmt);
}

#if defined (PIB_TABCONTROL_ENABLE_THEME) // [
static PibControl2 *_pib_tabcontrol_create_ex(PibControl2 *control, HWND hDlg)
{
  EnableThemeDialogTexture(
    control->hDlg,                // _In_ HWND  hwnd,
    ETDT_ENABLETAB                // _In_ DWORD dwFlags
  );

  return control;
}
#endif // ]

static void _pib_tabcontrol_set(PibControl2 *control, int n)
{
  enum {
    SWP_FLAGS=SWP_NOACTIVATE
        |SWP_NOMOVE
        |SWP_NOSIZE
        |SWP_NOZORDER
  };

  int *np=&control->branch.tabcontrol.npage;

  if (*np!=n) {
    SetWindowPos(
      control->branch.children[*np]->hDlg,  // HWND hWnd,
      NULL,                                 // HWND hWndInsertAfter,
      0,                                    // int  X,
      0,                                    // int  Y,
      0,                                    // int  cx,
      0,                                    // int  cy,
      SWP_FLAGS|SWP_HIDEWINDOW              // UINT uFlags
    );

    *np=n;

    SetWindowPos(
      control->branch.children[*np]->hDlg,  // HWND hWnd,
      NULL,                                 // HWND hWndInsertAfter,
      0,                                    // int  X,
      0,                                    // int  Y,
      0,                                    // int  cx,
      0,                                    // int  cy,
      SWP_FLAGS|SWP_SHOWWINDOW              // UINT uFlags
    );
  }
}

static void _pib_tabcontrol_adjust(PibControl2 *control, PibControl2 *parent)
{
  int npage;

  /////////////////////////////////////////////////////////////////////////////
  control->tabcontrol.property.npage
      =pib_control2_property(control,control->param->tabcontrol.index);

  if (!control->tabcontrol.property.npage) {
    DMESSAGE("missing npage property");
    return;
  }

  /////////////////////////////////////////////////////////////////////////////
  npage=pib_property_get_int(control->tabcontrol.property.npage,
      control->properties->cons);

  TabCtrl_SetCurSel(control->hDlg,npage);
  _pib_tabcontrol_set(control,npage);
}

static void _pib_tabcontrol_notify(PibControl2 *control, WPARAM wParam,
    LPARAM lParam)
{
  void *trans=control->properties->trans;
  PibProperty *npage=control->tabcontrol.property.npage;

  _pib_tabcontrol_set(control,TabCtrl_GetCurSel(control->hDlg));
  pib_property_set_int(npage,trans,control->branch.tabcontrol.npage);
  pib_property_write(npage,control->properties);
}

static PibControlVmt2 vmt={
  .id="TABCONTROL",
  .type=PIB_CONTROL_TYPE_TABCONTROL,
#if defined (PIB_TABCONTROL_ENABLE_THEME) // [
  .create_ex=_pib_tabcontrol_create_ex,
#endif // ]
  .destroy=_pib_branch_destroy,
  .refresh=_pib_branch_refresh,
  .adjust=_pib_tabcontrol_adjust,
  .notify=_pib_tabcontrol_notify,
};
