/*
 * pib_control2_tabpage.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <pib.h>
#include <uxtheme.h>

static PibControlVmt2 vmt;

PibControl2 *pib_tabpage_create(HWND hDlg, PibControl2 *parent,
    PibControlParam2 *param, PibModel2 *model)
{
  return pib_branch_create(hDlg,parent,1,param,model,&vmt);
}

static PibControl2 *_pib_tabpage_create_ex(PibControl2 *control, HWND hDlg)
{
  EnableThemeDialogTexture(
    control->hDlg,                // _In_ HWND  hwnd,
    ETDT_ENABLETAB                // _In_ DWORD dwFlags
  );

  return control;
}

static void _pib_tabpage_adjust(PibControl2 *control, PibControl2 *parent)
{
  TCITEMW tci={
    .mask=TCIF_TEXT,
    .pszText=control->param->branch.label,
  };

  int *nchildren=&parent->branch.tabcontrol.nchildren;
  UINT uFlags=parent->branch.tabcontrol.npage==*nchildren
      ?SWP_SHOWWINDOW:SWP_HIDEWINDOW;
  RECT rcParent,rcChild;
  int bResult;

  SendMessageW(parent->hDlg,TCM_INSERTITEMW,*nchildren,(LPARAM)&tci);

  // get the tabcontrol rect.
  GetClientRect(
    parent->hDlg,                 // HWND   hWnd,
    &rcParent                     // LPRECT lpRect
  );

  // adjust the tabcontrol rect to fit into the tabpage area.
  TabCtrl_AdjustRect(
    parent->hDlg,                 // hwnd,
    FALSE,                        // bLarger,
    &rcParent                     // prc
  );

  rcParent.right+=4;
  rcParent.top+=2;
  rcParent.bottom+=5;

  // get the tabpage rect.
  GetClientRect(
    control->hDlg,                // HWND   hWnd,
    &rcChild                      // LPRECT lpRect
  );

  // center the tabpage rect into the tabcontrol rect.
  _pib_branch_rect_center(&rcParent,&rcChild,&rcChild);

  bResult=SetWindowPos(
    control->hDlg,                // HWND hWnd,
    NULL,                         // HWND hWndInsertAfter,
    rcChild.left,                 // int  X,
    rcChild.top,                  // int  Y,
    rcChild.right-rcChild.left,   // int  cx,
    rcChild.bottom-rcChild.top,   // int  cy,
    SWP_NOZORDER|uFlags           // UINT uFlags
  );

  if (!bResult)
    DVMESSAGE("position window failed: %lu",GetLastError());

  ++*nchildren;
}

static PibControlVmt2 vmt={
  .id="TABPAGE",
  .type=PIB_CONTROL_TYPE_TABPAGE,
  .create_ex=_pib_tabpage_create_ex,
  .destroy=_pib_branch_destroy,
  .refresh=_pib_branch_refresh,
  .adjust=_pib_tabpage_adjust,
};
