/*
 * yapib.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yapib.h>

///////////////////////////////////////////////////////////////////////////////
enum {
  YAPIB_NPROPERTIES
      =(sizeof ((Yapib *)0)->property)/(sizeof ((Yapib *)0)->property.min[0])
}; 

static PibModelVmt2 vmt;

///////////////////////////////////////////////////////////////////////////////
int yapib_create(Yapib *yapib, WNDPROC wndproc)
{
  PibProperty *min=yapib->property.min;
  PibProperty *max=min+YAPIB_NPROPERTIES;

  ////////
  memset(yapib,0,sizeof *yapib);
  yapib->vmt=&vmt;

  ////////
  if (pib_create_ini_path(&yapib->ini.path,GEN_YAPIBW)<0) {
    DMESSAGE("allocating path");
    goto e_path;
  }

DVWRITELNW("\"%" PIB_PRIsw L"\"",yapib->ini.path);

  // consolidated variables:
  yapib->variables.cons.x=-1l;
  yapib->variables.cons.y=-1l;

  // transient variables:
  yapib->variables.trans=yapib->variables.cons;

  // peristent properties:
  pib_property_long_create(&yapib->property.x,L"geometry",L"x",
      offsetof(YapibVariables,x));
  pib_property_long_create(&yapib->property.y,L"geometry",L"y",
      offsetof(YapibVariables,y));

  ////////
DVWRITELNW("\"%" PIB_PRIsw L"\"",yapib->ini.path);
  if (pib_properties_create(&yapib->properties,min,max,&yapib->variables.cons,
      &yapib->variables.trans,yapib->ini.path)<0) {
    DMESSAGE("creating properties");
    goto e_properties;
  }

#if defined (YAPIB_LIST) // [
  ////////
  if (yapib_list_create(&yapib->list)<0) {
    DMESSAGE("creating list");
    goto e_list;
  }
#endif // ]

  ////////
  if (wndproc) {
    yapib->wndproc.wa=(WNDPROC)SetWindowLongW(
      plugin_hwnd_main(),       // _In_ HWND hWnd,
      GWL_WNDPROC,              // _In_ int  nIndex,
      (LONG)wndproc             // _In_ LONG dwNewLong
    );
  }

  ////////
  return 0;
//cleanup:
#if defined (YAPIB_LIST) // [
  yapib_list_destroy(&yapib->list);
e_list:
#endif // ]
  pib_properties_destroy(&yapib->properties);
e_properties:
  pib_destroy_ini_path(&yapib->ini.path);
e_path:
  return -1;
}

void yapib_destroy(Yapib *yapib)
{
  if (yapib->wndproc.wa) {
    SetWindowLongW(
      plugin_hwnd_main(),       // _In_ HWND hWnd,
      GWL_WNDPROC,              // _In_ int  nIndex,
      (LONG)yapib->wndproc.wa   // _In_ LONG dwNewLong
    );
  }

#if defined (YAPIB_LIST) // [
  yapib_list_destroy(&yapib->list);
#endif // ]
  pib_properties_destroy(&yapib->properties);
  pib_destroy_ini_path(&yapib->ini.path);
}

///////////////////////////////////////////////////////////////////////////////
static PibControl2 *yapib_root(PibModel2 *model)
{
  return ((Yapib *)model)->form;
}

static PibProperties *yapib_properties(PibModel2 *model)
{
  return &((Yapib *)model)->properties;
}

static PibProperty *yapib_property(PibModel2 *model, int index)
{
  PibProperty *min=((Yapib *)model)->property.min;
  PibProperty *max=min+YAPIB_NPROPERTIES;
  PibProperty *property=min+index;

//DVWRITELN("%d (%d)",index,YAPIB_PROPERTY_MAX);

  if (YAPIB_PROPERTY_MAX<=index) {
    DVMESSAGE("index [%d] out of range (%d)",index,YAPIB_PROPERTY_MAX);
    goto e_property;
  }
  else if (property<min&&max<=property) {
    DVMESSAGE("index [%d] out of range",index);
    goto e_property;
  }
  else if (!property->vmt) {
    DVMESSAGE("missing property->vmt (index:%d)",index);
    goto e_property;
  }

  return property;
//cleanup:
e_property:
  return NULL;
}

static PibModelVmt2 vmt={
  .root=yapib_root,
  .properties=yapib_properties,
  .property=yapib_property,
  .commit=pib_model_commit,
  .rollback=pib_model_rollback,
};
