/*
 * yapib.h
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#if ! defined (__YAPIB_H__) // [

#include <stdio.h>
#include <windows.h>
#include <Winamp/gen.h>
#include <pib.h>

///////////////////////////////////////////////////////////////////////////////
#define GEN_YAPIBW      L"gen_yapib"
//#define YAPIB_WIDTH     8

#if ! defined (YAPIB_VERSION_MAJOR) // [
#define YAPIB_VERSION_MAJOR 0
#endif // ]
#if ! defined (YAPIB_VERSION_MINOR) // [
#define YAPIB_VERSION_MINOR 0
#endif // ]
#if ! defined (YAPIB_VERSION_BUILD) // [
#define YAPIB_VERSION_BUILD 0
#endif // ]

#define YAPIB_VERSIONA \
		PIB_STRA(YAPIB_VERSION_MAJOR.YAPIB_VERSION_MINOR.YAPIB_VERSION_BUILD)

#if defined (__GNUC__) // [
#define YAPIB_LABELA    "[Y]et [A]nother [P]lug-[i]n [B]undle (YAPIB/GCC) v"
#elif defined (_MSC_VER) // ] [
#define YAPIB_LABELA    "[Y]et [A]nother [P]lug-[i]n [B]undle (YAPIB/MSC) v"
#else // ] [
#define YAPIB_LABELA    "[Y]et [A]nother [P]lug-[i]n [B]undle (YAPIB) v"
#endif // ]

#if defined (YAPIB_VERSION_BETA) // [
#define YAPIB_BETAA     "-beta" PIB_STRA(YAPIB_VERSION_BETA)
/*
 * Complete Character List for UTF-16
 *   GREEK SMALL LETTER BETA (U+03B2)
 *   http://www.fileformat.info/info/charset/UTF-16/list.htm
 */
#define YAPIB_BETAW     L"-\x03b2" PIB_STRW(YAPIB_VERSION_BETA)
#else // ] [
#define YAPIB_BETAA     ""
#define YAPIB_BETAW     L""
#endif // ]
#define YAPIB_TITLEA    YAPIB_LABELA YAPIB_VERSIONA YAPIB_BETAA
#define YAPIB_TITLEW    PIB_WIDEN(YAPIB_LABELA) PIB_WIDEN(YAPIB_VERSIONA) \
                        YAPIB_BETAW

///////////////////////////////////////////////////////////////////////////////
#if defined (YAPIB_LIST) // [
typedef struct _YapibList YapibList;
#endif // ]
typedef struct _YapibVariables YapibVariables;
typedef union _YapibProperty YapibProperty;
typedef enum _YapibPropertyOffs YapibPropertyOffs;
typedef struct _Yapib Yapib;

#if defined (YAPIB_LIST) // [
///////////////////////////////////////////////////////////////////////////////
#if 0 // [
struct _YapibNode {
  YapibNode *prev,*next;
#if defined (YAPIB_NODE_FORM) // [
  PibForm *form;
#else // ] [
  PibFormInit *init;
  HWND hDlg;
#endif // ]
};
#endif // ]

#if defined (YAPIB_NODE_FORM) // [
YapibNode *yapib_node_create(PibForm *form);
#else // ] [
YapibNode *yapib_node_create(PibFormInit *init);
#endif // ]
void yapib_node_destroy(YapibNode *node);

////////
struct _YapibList {
  YapibNode *head,*tail;
  int count;
};

int yapib_list_create(YapibList *list);
void yapib_list_destroy(YapibList *list);

YapibNode *yapib_list_unlink(YapibList *list, YapibNode *node);
YapibNode *yapib_list_pop(YapibList *list);

#if defined (YAPIB_NODE_FORM) // [
YapibNode *yapib_list_insert(YapibList *list, PibForm *form);
#else // ] [
YapibNode *yapib_list_insert(YapibList *list, PibFormInit *init);
#endif // ]
#endif // ]

///////////////////////////////////////////////////////////////////////////////
struct _YapibVariables {
  long x,y;
};

union _YapibProperty {
  struct {
    PibProperty x;
    PibProperty y;
  };

  PibProperty min[1];
};

enum _YapibPropertyOffs {
  YAPIB_PROPERTY_ELEMENT=sizeof ((YapibProperty *)0)->min[0],
  YAPIB_PROPERTY_X=offsetof(YapibProperty,x)/YAPIB_PROPERTY_ELEMENT,
  YAPIB_PROPERTY_Y=offsetof(YapibProperty,y)/YAPIB_PROPERTY_ELEMENT,
  YAPIB_PROPERTY_MAX=sizeof (YapibProperty)/YAPIB_PROPERTY_ELEMENT,
};

#if 0 // [
#define yas_form_cons(form) \
  ((YapibVariables *)(form)->properties->cons)
#define yas_form_trans(form) \
  ((YapibVariables *)(form)->properties->trans)
#define yas_form_yas(form) \
  ((Yapib *)(form)->data)

union _YapibProperty {
  PibProperty min[1];

  struct {
    PibProperty x;
    PibProperty y;
  };
};

enum _YapibPropertyOffs {
  YAPIB_PROPERTY_X=offsetof(YapibProperty,x),
  YAPIB_PROPERTY_Y=offsetof(YapibProperty,y),
};
#endif // ]

struct _Yapib {
  PibModelVmt2 *vmt;

  struct {
    wchar_t *path;
  } ini;

  struct {
    YapibVariables cons,trans;
  } variables;

  YapibProperty property;
  PibProperties properties;
#if defined (PIB_CONTROL_DISABLED) // [
  PibControl2 *form;
#else // ] [
  HWND hDlg;
#endif // ]
#if defined (YAPIB_LIST) // [
  YapibList list;
#endif // ]

  struct {
    WNDPROC wa;
  } wndproc;
};

int yapib_create(Yapib *yapib, WNDPROC wndproc);
void yapib_destroy(Yapib *yapib);

#define yapib_cons(yapib) \
  ((YapibVariables *)(yapib)->properties.cons)
#define yapib_trans(yapib) \
  ((YapibVariables *)(yapib)->properties.trans)
#if 0 // [
#define yas_form_yas(form) \
  ((Yapib *)(form)->data)
#endif // ]

///////////////////////////////////////////////////////////////////////////////
extern PibControlParam2 yapib_form_main;

///////////////////////////////////////////////////////////////////////////////
__declspec( dllexport )
winampGeneralPurposePlugin *winampGetGeneralPurposePlugin();

#endif // ]
