/*
 * yapib_list.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yapib.h>

#if defined (YAPIB_LIST) // [
///////////////////////////////////////////////////////////////////////////////
#if defined (YAPIB_NODE_FORM) // [
YapibNode *yapib_node_create(PibForm *form)
#else // ] [
YapibNode *yapib_node_create(PibFormInit *init)
#endif // ]
{
  YapibNode *node=malloc(sizeof *node);

  if (!node) {
    DMESSAGEW("allocating node");
    goto e_node;
  }

  node->prev=NULL;
  node->next=NULL;
#if defined (YAPIB_NODE_FORM) // [
  node->form=form;
#else // ] [
  node->init=init;
  node->hDlg=NULL;
#endif // ]

  return node;
//cleanup:
  free(node);
e_node:
  return NULL;
}

void yapib_node_destroy(YapibNode *node)
{
  free(node);
}

///////////////////////////////////////////////////////////////////////////////
int yapib_list_create(YapibList *list)
{
  list->head=NULL;
  list->tail=NULL;
  list->count=0;

  return 0;
}

void yapib_list_destroy(YapibList *list)
{
  YapibNode *cur,*prev;

  cur=list->tail;

  while (cur) {
    prev=cur->prev;
    yapib_list_unlink(list,cur);
    yapib_node_destroy(cur);
    cur=prev;
  }
}

YapibNode *yapib_list_unlink(YapibList *list, YapibNode *node)
{
  YapibNode *prev=node->prev;
  YapibNode *next=node->next;

  if (prev)
    prev->next=node->next;
  else
    list->head=node->next;

  node->next=NULL;

  if (next)
    next->prev=node->prev;
  else
    list->tail=node->prev;

  node->prev=NULL;
  --list->count;

  return prev;
}

YapibNode *yapib_list_pop(YapibList *list)
{
  YapibNode *node=list->tail;

  yapib_list_unlink(list,node);

  return node;
}

#if 0 // [
YapibNode *yapib_list_push(YapibList *list, YapibNode *node)
{
  node->prev=list->tail;
  node->next=NULL;

  if (list->tail)
    list->tail->next=node;
  else
    list->head=node;

  ++list->count;

  return list->tail=node;
}
#endif // ]

#if defined (YAPIB_NODE_FORM) // [
#define yapib_node_cmp(lhs,rhs) \
  wcscmp(pib_form_label((lhs)->form),pib_form_label((rhs)->form))
#elif defined (YAPIB_NODE_CMP_INIT) // ] [
#define yapib_node_cmp(lhs,rhs) \
  wcscmp((lhs)->init->form_vmt->label,(rhs)->init->form_vmt->label)
#else // ] [
#define yapib_node_cmp(lhs,rhs) \
  wcscmp((lhs)->form_vmt->label,(rhs)->form_vmt->label)
#endif // ]

/*
 * https://www.geeksforgeeks.org/insert-value-sorted-way-sorted-doubly-linked-list/
 */
#if defined (YAPIB_NODE_FORM) // [
YapibNode *yapib_list_insert(YapibList *list, PibForm *form)
#else // ] [
YapibNode *yapib_list_insert(YapibList *list, PibFormInit *init)
#endif // ]
{
#if defined (YAPIB_NODE_FORM) // [
  YapibNode *node=yapib_node_create(form);

  if (!node) {
    DMESSAGEW("creating node");
    goto e_node;
  }
#elif defined (YAPIB_NODE_CMP_INIT) // ] [
  YapibNode *node=yapib_node_create(init);

  if (!node) {
    DMESSAGEW("creating node");
    goto e_node;
  }
#else // ] [
  YapibNode *node;
#endif // ]

  if (!list->head) {
    // if list is empty
#if ! defined (YAPIB_NODE_FORM) && ! defined (YAPIB_NODE_CMP_INIT) // [
    node=yapib_node_create(init);

    if (!node) {
      DMESSAGEW("creating node");
      goto e_node;
    }
#endif // ]
    list->head=node;
    list->tail=node;
  }
  else if (yapib_node_cmp(init,list->head->init)<0) {
    // if the node is to be inserted at the beginning
    // of the doubly linked list
#if ! defined (YAPIB_NODE_FORM) && ! defined (YAPIB_NODE_CMP_INIT) // [
    if (init==list->head->init) {
      node=list->head;
      goto e_success;
    }

    node=yapib_node_create(init);

    if (!node) {
      DMESSAGEW("creating node");
      goto e_node;
    }
#endif // ]
    node->next=list->head; 
    node->next->prev=node; 
    list->head=node;
  }
  else {
    YapibNode *cur=list->head;

    // locate the node after which the new node
    // is to be inserted
    while (cur->next&&yapib_node_cmp(cur->next->init,init)<0)
      cur=cur->next;

#if ! defined (YAPIB_NODE_FORM) && ! defined (YAPIB_NODE_CMP_INIT) // [
    if (init==cur->init) {
      node=cur;
      goto e_success;
    }

    node=yapib_node_create(init);

    if (!node) {
      DMESSAGEW("creating node");
      goto e_node;
    }
#endif // ]

    // make the appropriate links
    node->next=cur->next;

    // if the new node is not inserted at the end of the list
    if (cur->next)
      node->next->prev=node;
    else
      list->tail=node;

    cur->next=node;
    node->prev=cur;
  }

  ++list->count;
#if ! defined (YAPIB_NODE_FORM) && ! defined (YAPIB_NODE_CMP_INIT) // [
e_success:
#endif // ]
  return node;
//cleanup:
  free(node);
e_node:
  return node;
}
#endif // ]
