/*
 * gen_yapib.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yapib.h>
#include <yapib_res/resource.h>

#if defined (YAPIB_LIST) // [
///////////////////////////////////////////////////////////////////////////////
static void config();
#endif // ]

///////////////////////////////////////////////////////////////////////////////
static Yapib yapib;

#if 0 // [
void CenterInsideParent(HWND hWnd)
{
    /* start calculating in parent's coordinates */
    HWND hParent=GetParent(hWnd);
    RECT rcp, rcc;
    GetClientRect(hParent, &rcp);
    GetWindowRect(hWnd, &rcc);

    /* offset child rect to top left */
    rcc.bottom -= rcc.top; /* height */
    rcc.right -= rcc.left; /* width */
    rcc.left=rcc.top=0;

    /* get top-left anchor */
    rcc.left=(rcp.right - rcc.right)/2;
    rcc.top=(rcp.bottom - rcc.bottom)/2;

    /* non-child to be centered in screen coordinates, not in parent's ones */
    if (!(GetWindowLong(hWnd, GWL_STYLE) & WS_CHILD))
    {
        /* adjust anchor position */
        MapWindowPoints(hParent, GetDesktopWindow(), (LPPOINT)&rcc, 1);
    }

    //MoveWindow(hWnd, rcc.left, rcc.top, rcc.right /*width*/, rcc.bottom /*height*/, TRUE);
    SetWindowPos(hWnd, NULL, rcc.left, rcc.top, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
}
#endif // ]

#if defined (YAPIB_LIST) // [
/*
 * Thread: [RESOLVED] win32 - how center a control depending on it's parent
 * position and size?
 * https://forums.codeguru.com/showthread.php?551357-RESOLVED-win32-how-center-a-control-depending-on-it-s-parent-position-and-size
 */
static void yapib_adjust(HWND hWnd)
{
  enum { DELTA=8 };
  /* start calculating in parent's coordinates */
  HWND hParent=GetParent(hWnd);
  RECT rcParent,rcTmp,rcChild;

  GetClientRect(hParent,&rcParent);
  GetClientRect(GetDlgItem(hParent,IDC_TREEVIEW_PLUGIN),&rcTmp);
DVWRITELNW(">>>>>>>>>>>> left:%d",rcParent.left);
  rcTmp.right+=DELTA;
  rcParent.left+=rcTmp.right;
  rcParent.right-=rcTmp.right;
DVWRITELNW("<<<<<<<<<<<< left:%d",rcParent.left);
  GetClientRect(GetDlgItem(hParent,IDC_BUTTON_ROLLBACK),&rcTmp);
DVWRITELNW(">>>>>>>>>>>> bottom:%d",rcParent.bottom);
  //rcTmp.bottom+=DELTA;
  rcParent.bottom-=rcTmp.bottom;
DVWRITELNW("<<<<<<<<<<<< bottom:%d",rcParent.bottom);
  GetWindowRect(hWnd,&rcChild);

  /* offset child rect to top left */
  rcChild.bottom-=rcChild.top; /* height */
  rcChild.right-=rcChild.left; /* width */
  rcChild.left=rcChild.top=0;

  /* get top-left anchor */
  rcChild.left=rcParent.left+(rcParent.right-rcChild.right)/2;
  rcChild.top=(rcParent.bottom-rcChild.bottom)/2;

  /* non-child to be centered in screen coordinates, not in parent's ones */
  if (!(GetWindowLong(hWnd, GWL_STYLE)&WS_CHILD)) {
      /* adjust anchor position */
      MapWindowPoints(hParent,GetDesktopWindow(),(LPPOINT)&rcChild,1);
  }

  SetWindowPos(hWnd,NULL,rcChild.left,rcChild.top,0,0, SWP_NOSIZE|SWP_NOZORDER);
}
#endif // ]

//#define YAPIB_DIALOG_HELLO
static LRESULT CALLBACK YapibWaProc(HWND hWnd, UINT uMsg, WPARAM wParam,
    LPARAM lParam)
{
  enum {
    SWP_FLAGS=SWP_DEFERERASE
        |SWP_NOSENDCHANGING
        |SWP_NOACTIVATE
        |SWP_NOSIZE
        |SWP_NOZORDER
        |SWP_NOREDRAW
  };

  switch (uMsg) {
#if defined (YAPIB_LIST) // [
  case WM_YAPIB_PRESENT: {
      PibCreateParam *param=(PibCreateParam *)lParam;
      YapibNode *node;

      config();
      node=yapib_list_insert(&yapib.list,param->init);

      if (!node)
        return 0;

      if (!node->hDlg) {
				node->hDlg=param->dialog_create(yapib.hDlg,param);
        yapib_adjust(node->hDlg);
			}

      return (LRESULT)node;
    }
#if 0 // [
  case WM_YAPIB_REGISTER_REQUEST:
    return 0;
#endif // ]
#endif // ]
  default:
    break;
  }

  return CallWindowProcW(
    yapib.wndproc.wa,             // _In_ WNDPROC lpPrevWndFunc,
    hWnd,                         // _In_ HWND    hWnd,
    uMsg,                         // _In_ UINT    Msg,
    wParam,                       // _In_ WPARAM  wParam,
    lParam                        // _In_ LPARAM  lParam
  );
}

#if ! defined (PIB_CONTROL_DISABLED) // [
///////////////////////////////////////////////////////////////////////////////
static INT_PTR CALLBACK YapibDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam,
    LPARAM lParam)
{
  Yapib *yapib=(Yapib *)GetWindowLongPtrW(hDlg,GWLP_USERDATA);

  switch (uMsg) {
  case WM_INITDIALOG:
    SetWindowLongPtrW(hDlg,GWLP_USERDATA,lParam);
    yapib=(Yapib *)lParam;

    SetWindowTextW(
      hDlg,                       // HWND   hWnd,
#if 0 // [
      L"Configure " YAPIB_TITLEW  // LPCSTR lpString
#else // ] [
      YAPIB_TITLEW                // LPCSTR lpString
#endif // ]
    );

    if (0<=yapib_cons(yapib)->x&&0<=yapib_cons(yapib)->y) {
      SetWindowPos(
        hDlg,                     // HWND hWnd,
        HWND_TOPMOST,             // HWND hWndInsertAfter,
        yapib_cons(yapib)->x,     // int  X,
        yapib_cons(yapib)->y,     // int  Y,
        0,                        // int  cx,
        0,                        // int  cy,
        SWP_NOZORDER|SWP_NOSIZE   // UINT uFlags
      );
    }

    break;
  case WM_COMMAND:
    switch(LOWORD(wParam)) {
    case IDOK:
      ShowWindow(hDlg,FALSE);
      return TRUE;
    case IDCANCEL:
      ShowWindow(hDlg,FALSE);
      return TRUE;
    default:
      break;
    }

    break;
  case WM_DESTROY: {
      RECT rect;

      GetWindowRect(hDlg,&rect);
      yapib_trans(yapib)->x=rect.left;
      yapib_trans(yapib)->y=rect.top;
      pib_properties_write(&yapib->properties);
    }

    break;
  default:
    break;
  }

  return FALSE;
}
#endif // ]


///////////////////////////////////////////////////////////////////////////////
static int init()
{
  INITCOMMONCONTROLSEX iccex={
    .dwSize=sizeof iccex,
    .dwICC=ICC_TAB_CLASSES,
  };

//#if defined (DEBUG) // [
//#error 55
//#endif // ]
  DCREATE();
  InitCommonControlsEx(&iccex); 

  if (yapib_create(&yapib,YapibWaProc)<0)
    goto e_yapib;

  return 0;
//cleanup:
  yapib_destroy(&yapib);
e_yapib:
  DDESTROY();
  return -1;
}

static void quit()
{
  yapib_destroy(&yapib);
  DDESTROY();
}

static void config()
{
#if defined (PIB_CONTROL_DISABLED) // [
  if (!yapib.form) {
    yapib.form=pib_form_create(pib_hwnd_parent(),&yapib_form_main,
        (PibModel2 *)&yapib);

    if (!yapib.form)
      DMESSAGE("creating mask");
  }
	else
    ShowWindow(yapib.form->hDlg,TRUE);
#else // ] [
//DVWRITELNW("!!!!!!!!!!!!!!!!!!!!!!!! %p",yapib.hDlg);
  if (!yapib.hDlg) {
#if 0 // [
    PibFormInit init={
      .form_vmt=&yapib_form_main_vmt,
      .form=&yapib.form.main,
      // the following is propagated to all sub-forms:
      .properties=&yapib.properties,
      .data=&yapib,
    };
#endif // ]

    yapib.hDlg=CreateDialogParamW(
      plugin_hinstance(),                 // HINSTANCE hInstance,
      MAKEINTRESOURCEW(IDD_CONFIG_YAPIB), // LPCSTR    lpTemplateName,
      pib_hwnd_parent(),                  // HWND      hWndParent,
#if 0 // [
      init.form_vmt->dlgproc,             // DLGPROC   lpDialogFunc,
#else // ] [
      YapibDlgProc,                        // DLGPROC   lpDialogFunc,
#endif // ]
#if 0 // [
      (LPARAM)&init                       // LPARAM    dwInitParam
#else // ] [
      (LPARAM)&yapib                      // LPARAM    dwInitParam
#endif // ]
    );
//DVWRITELNW("!!!!!!!!!!!!!!!!!!!!!!!! %p",yapib.hDlg);
  }
  else if (!IsWindowVisible(yapib.hDlg))
    ShowWindow(yapib.hDlg,TRUE);
#endif // ]
}

///////////////////////////////////////////////////////////////////////////////
HINSTANCE plugin_hinstance(void)
{
  return winampGetGeneralPurposePlugin()->hDllInstance;
}

const char *plugin_titlea(void)
{
   return YAPIB_TITLEA;
}

const wchar_t *plugin_titlew(void)
{
   return YAPIB_TITLEW;
}

HWND plugin_hwnd_main(void)
{
  return winampGetGeneralPurposePlugin()->hwndParent;
}

///////////////////////////////////////////////////////////////////////////////
__declspec( dllexport )
winampGeneralPurposePlugin *winampGetGeneralPurposePlugin()
{
  static winampGeneralPurposePlugin p={
    GPPHDR_VER_U,
    (char *)YAPIB_TITLEW,
    init,
    config,
    quit
  };

  return &p;
}
