/*
 * yas.h
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#ifndef __YAS_H__ // [
#define __YAS_H__

#include <sfmt.h>
#include <pib.h>

///////////////////////////////////////////////////////////////////////////////
// DEBUG via "pib.h"!
//#define PLUGIN_PATH_PTR
//#define YAS_ACTIVE_SYNC
#define YAS_PILOT
#define YAS_NEG
#define YAS_SWAP_HISTOGRAM
#define YAS_FORM_ALGORITHMS_VMT
//#define GEN_YAS_WINDOW
//#define OPTIONS2
//#define SHOWWINDOW

//#define YAS_RAND_SHUFFLE_LOOP

///////////////////////////////////////////////////////////////////////////////
#define GEN_YASW      L"gen_yas"
#define YAS_WIDTH     8

#if ! defined (YAS_VERSION_MAJOR) // [
#define YAS_VERSION_MAJOR 0
#endif // ]
#if ! defined (YAS_VERSION_MINOR) // [
#define YAS_VERSION_MINOR 0
#endif // ]
#if ! defined (YAS_VERSION_BUILD) // [
#define YAS_VERSION_BUILD 0
#endif // ]

#define YAS_VERSIONA \
		PIB_STRA(YAS_VERSION_MAJOR.YAS_VERSION_MINOR.YAS_VERSION_BUILD)

#if defined (__GNUC__) // [
#define YAS_LABELA    "[Y]et [A]nother [S]huffle (YAS/GCC) v"
#elif defined (_MSC_VER) // ] [
#define YAS_LABELA    "[Y]et [A]nother [S]huffle (YAS/MSC) v"
#else // ] [
#define YAS_LABELA    "[Y]et [A]nother [S]huffle (YAS) v"
#endif // ]

#if defined (YAS_VERSION_BETA) // [
#define YAS_BETAA     "-beta" PIB_STRA(YAS_VERSION_BETA)
/*
 * Complete Character List for UTF-16
 *   GREEK SMALL LETTER BETA (U+03B2)
 *   http://www.fileformat.info/info/charset/UTF-16/list.htm
 */
#define YAS_BETAW     L"-\x03b2" PIB_STRW(YAS_VERSION_BETA)
#else // ] [
#define YAS_BETAA     ""
#define YAS_BETAW     L""
#endif // ]
#define YAS_TITLEA    YAS_LABELA YAS_VERSIONA YAS_BETAA
#define YAS_TITLEW    PIB_WIDEN(YAS_LABELA) PIB_WIDEN(YAS_VERSIONA) YAS_BETAW

///////////////////////////////////////////////////////////////////////////////
typedef struct _YasHistogramBin YasHistogramBin;
typedef enum _YasHistogramParam YasHistogramParam;
typedef struct _YasHistogram YasHistogram;
typedef const struct _YasRandVmt YasRandVmt;
typedef struct _YasRand YasRand;
typedef struct _YasSync YasSync;
typedef union _YasProperty YasProperty;

typedef enum _YasRandMode YasRandMode;
typedef struct _YasVariables YasVariables;
#if defined (YAPIB_LIST) // [
typedef enum _YasMode YasMode;
#endif // ]
typedef struct _Yas Yas;
typedef struct _YasThreadSyncRequestCreate YasThreadSyncRequestCreate;
typedef struct _YasThreadSyncRequestUpdate YasThreadSyncRequestUpdate;

typedef uint32_t yas_rand_t;

///////////////////////////////////////////////////////////////////////////////
struct _YasHistogramBin {
  yas_rand_t base,incr;
};

enum _YasHistogramParam {
  YAS_HIST_NBINS=100,
};

struct _YasHistogram {
  size_t nbins;
  yas_rand_t max;
  YasHistogramBin bin[YAS_HIST_NBINS];
};

int yas_histogram_create(YasHistogram *hist);
void yas_histogram_destroy(YasHistogram *hist);

void yas_histogram_reset(YasHistogram *hist, yas_rand_t nbins);
int yas_histogram_index(YasHistogram *hist, yas_rand_t pos, yas_rand_t size);
void yas_histogram_base_add(YasHistogram *hist, yas_rand_t pos,
    yas_rand_t size);
void yas_histogram_incr_add(YasHistogram *hist, yas_rand_t pos,
    yas_rand_t size);

///////////////////////////////////////////////////////////////////////////////
struct _YasRandVmt {
  char *id;
  yas_rand_t max;
  void (*destroy)(YasRand *rand);
  int (*next)(YasRand *rand, yas_rand_t *result);
};

struct _YasRand {
  YasRandVmt *vmt;

  union {
    sfmt_t sfmt;
    HCRYPTPROV hProv;

    struct {
      yas_rand_t state[16];
      uint16_t index;
    } well;

    struct {
      yas_rand_t c;
      yas_rand_t Q[4096];
      uint16_t i;
    } marsaglia_cmwc4096;
  };
};

int yas_rand_crypt_create(YasRand *rand);
int yas_rand_well_create(YasRand *rand);
int yas_rand_sfmt_create(YasRand *rand);
int yas_rand_libc_create(YasRand *rand);
int yas_rand_marsaglia_cmwc4096_create(YasRand *rand);

#if defined (YAS_RAND_SHUFFLE_LOOP) // [
int yas_rand_shuffle_loop(YasRand *rand, yas_rand_t *array, yas_rand_t size,
    int nloops);
#else // ] [
int yas_rand_shuffle(YasRand *rand, yas_rand_t *array, yas_rand_t size);
#endif // ]

///////////////////////////////////////////////////////////////////////////////
enum _YasRandMode {
  YAS_RAND_MODE_CRYPT,
  YAS_RAND_MODE_WELL,
  YAS_RAND_MODE_SFMT,
  YAS_RAND_MODE_LIBC,
  YAS_RAND_MODE_MARSAGLIA_CMWC4096,
  YAS_RAND_MODE_MAX,
};

struct _YasVariables {
  long x,y;
  int page;
  int active;

  int play;
  YasRandMode mode;
  int histogram;
};

union _YasProperty {
  struct {
    PibProperty x;
    PibProperty y;
    PibProperty page;
    PibProperty active;
    PibProperty play;
    PibProperty mode;
    PibProperty histogram;
  };

  PibProperty min[1];
};

enum _YasPropertyOffs {
  YAS_PROPERTY_ELEMENT=sizeof ((YasProperty *)0)->min[0],
  YAS_PROPERTY_X=offsetof(YasProperty,x)/YAS_PROPERTY_ELEMENT,
  YAS_PROPERTY_Y=offsetof(YasProperty,y)/YAS_PROPERTY_ELEMENT,
  // tabcontrol from "yas_control2_main.c".
  YAS_PROPERTY_PAGE=offsetof(YasProperty,page)/YAS_PROPERTY_ELEMENT,
  // checkbox "Active" from "yas_control2_config.c" page.
  YAS_PROPERTY_ACTIVE=offsetof(YasProperty,active)/YAS_PROPERTY_ELEMENT,
  // checkbox "Start playing from Prev/Next" on "yas_control2_config.c" page.
  YAS_PROPERTY_PLAY=offsetof(YasProperty,play)/YAS_PROPERTY_ELEMENT,
  YAS_PROPERTY_MODE=offsetof(YasProperty,mode)/YAS_PROPERTY_ELEMENT,
  // checkbox "Higlight session increments" from "yas_control2_monitor.c" page.
  YAS_PROPERTY_HISTOGRAM=offsetof(YasProperty,histogram)/YAS_PROPERTY_ELEMENT,
  YAS_PROPERTY_MAX=sizeof (YasProperty)/YAS_PROPERTY_ELEMENT,
};

#if defined (YAPIB_LIST) // [
enum _YasMode {
  YasModeNull,
  YasModeOff,
  YasModeOn,
};
#endif // ]

struct _Yas {
  PibModelVmt2 *vmt;

  struct {
    WNDPROC wa;
#if defined (RAND_SUBCLASS_PE) // [
    WNDPROC pe;
#endif // ]
#if defined (RAND_SUBCLASS_EQ) // [
    WNDPROC eq;
#endif // ]
  } wndproc;

  struct {
    wchar_t *path;
  } ini;

  struct {
    YasVariables cons,trans;
  } variables;

  YasProperty property;
  PibProperties properties;
  int (*yas_rand_create[YAS_RAND_MODE_MAX])(YasRand *rand);
#if defined (DEBUG) // [
  yas_rand_t pos;
  wchar_t file[MAX_PATH];
#endif // ]

  struct {
    wchar_t *path;
  } state;

  yas_rand_t *shuffle;
  yas_rand_t size;
  yas_rand_t cur;
#if defined (YAS_NEG) // [
  yas_rand_t neg;
#endif // ]
#if defined (YAS_PILOT) // [
  yas_rand_t pilot;
#endif // ]
  YasHistogram hist;

  PibThread sync;
  PibControl2 *form;
};

#define yas_cons(yas) \
  (&(yas)->variables.cons)
#define yas_trans(yas) \
  (&(yas)->variables.trans)

void yas_refresh(Yas *yas);

extern Yas yas;

int yas_create(Yas *yas, WNDPROC wndproc);
void yas_destroy(Yas *yas);

int yas_reload(Yas *yas, yas_rand_t size, YasRandMode mode);
yas_rand_t yas_next(Yas *yas);
yas_rand_t yas_prev(Yas *yas);

void yas_sync_create(Yas *yas);
void yas_sync_update(Yas *yas);

////////
struct _YasThreadSyncRequestCreate {
  PibThreadRequest common;

  struct {
    wchar_t *path;
    yas_rand_t size;
    yas_rand_t *shuffle;
    yas_rand_t cur;
#if defined (YAS_NEG) // [
    yas_rand_t neg;
#endif // ]
#if defined (YAS_PILOT) // [
    yas_rand_t pilot;
#endif // ]
  } yas;
};

YasThreadSyncRequestCreate *yas_thread_sync_request_create_new(Yas *yas);

////////
struct _YasThreadSyncRequestUpdate {
  PibThreadRequest common;

  struct {
    wchar_t *path;
    yas_rand_t cur;
#if defined (YAS_NEG) // [
    yas_rand_t neg;
#endif // ]
#if defined (YAS_PILOT) // [
    yas_rand_t pilot;
#endif // ]
  } yas;
};

YasThreadSyncRequestUpdate *yas_thread_sync_request_update_new(Yas *yas);

////////
extern PibControlParam2 yas_tabpage_config;
extern PibControlParam2 yas_tabpage_algorithms;
extern PibControlParam2 yas_tabpage_monitor;
extern PibControlParam2 yas_tabpage_license;
extern PibControlParam2 yas_tabpage_references;
extern PibControlParam2 yas_tabpage_notes;
////////
extern PibControlParam2 yas_form_main;

#if 0 // [
///////////////////////////////////////////////////////////////////////////////
__declspec (dllexport)
winampGeneralPurposePlugin *winampGetGeneralPurposePlugin();
#endif // ]

#endif // ]
