/*
 * yas_control2_main.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yas.h>
#include <yas_res/resource.h>

static void cb_proxy_parent_rect(PibControl2 *control, RECT *rcParent)
{
  HWND hDlg=control->parent->hDlg;
  RECT rcDelta;

  GetClientRect(
    hDlg,                   // HWND   hWnd,
    rcParent                // LPRECT lpRect
  );

  GetClientRect(
    GetDlgItem(hDlg,IDOK),  // HWND   hWnd,
    &rcDelta                // LPRECT lpRect
  );

DVWRITELN("bottom:%ld",rcParent->bottom);
  rcParent->bottom-=rcDelta.bottom;
DVWRITELN("bottom:%ld",rcParent->bottom);
}

void cb_button_rollback(PibControl2 *control)
{
  PibModel2 *model=control->model;

DVWRITELN("model:%p model->vmt:%p",model,model?model->vmt:NULL);
  model->vmt->rollback(model);
DMARKLN();
}

void cb_button_commit(PibControl2 *control)
{
  PibModel2 *model=control->model;

DVWRITELN("model:%p model->vmt:%p",model,model?model->vmt:NULL);
  model->vmt->commit(model);
DMARKLN();
}

void cb_button_cancel(PibControl2 *control)
{
  PibModel2 *model=control->model;

DVWRITELN("model:%p model->vmt:%p",model,model?model->vmt:NULL);
  model->vmt->rollback(model);
DMARKLN();
  ShowWindow(GetParent(control->hDlg),FALSE);
}

void cb_button_ok(PibControl2 *control)
{
  PibModel2 *model=control->model;

DVWRITELN("model:%p model->vmt:%p",model,model?model->vmt:NULL);
  model->vmt->commit(model);
DMARKLN();
  ShowWindow(GetParent(control->hDlg),FALSE);
}

static PibControlParam2 *yas_tabpages[]={
  &yas_tabpage_config,
  &yas_tabpage_algorithms,
  &yas_tabpage_monitor,
  &yas_tabpage_license,
  &yas_tabpage_references,
  &yas_tabpage_notes,
};

static PibControlParam2 tabcontrol
  =PIB_TABCONTROL(IDC_TAB_CONTROL_MAIN,yas_tabpages,YAS_PROPERTY_PAGE);
static PibControlParam2 *proxy_tabcontrol_params[]={ &tabcontrol, };
static PibControlParam2 proxy_tabcontrol=PIB_PROXY(IDD_CONFIG_TAB_CHILD,
    proxy_tabcontrol_params,cb_proxy_parent_rect);

static PibControlParam2 button_rollback
  =PIB_BUTTON(IDC_BUTTON_ROLLBACK,cb_button_rollback);
static PibControlParam2 button_commit
  =PIB_BUTTON(IDC_BUTTON_COMMIT,cb_button_commit);
static PibControlParam2 button_cancel
  =PIB_BUTTON(IDCANCEL,cb_button_cancel);
static PibControlParam2 button_ok
  =PIB_BUTTON(IDOK,cb_button_ok);

static PibControlParam2 *yas_form_main_params[]={
  &proxy_tabcontrol,
  &button_rollback,
  &button_commit,
  &button_cancel,
  &button_ok,
};

PibControlParam2 yas_form_main =PIB_FORM(IDD_CONFIG_FRAME,YAS_TITLEW L" #WLM",
    yas_form_main_params,YAS_PROPERTY_X,YAS_PROPERTY_Y);
