/*
 * yas_control2_monitor.c
 *
 * Copyright (C) 2020 Peter Belkner <pbelkner@users.sf.net>
 * Nanos gigantum humeris insidentes #TeamWhite
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.0 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yas.h>
#include <yas_res/resource.h>

///////////////////////////////////////////////////////////////////////////// {
#define YAS_PICTURE_NOTIFY
#define DIFF_BASE
//#define YAS_PICTURE_ROUND
//#define YAS_PICTURE_RECT
/*
 * GetSysColorBrush function
 * https://docs.microsoft.com/en-us/windows/win32/api/winuser/nf-winuser-getsyscolorbrush
 */
#if defined (YAS_PICTURE_RECT) // [
static void yas_image_callback(PibControl2 *control)
{
  Yas *yas=(Yas *)control->model;
  HWND hWnd=control->hWnd;
  RECT rect;
  double scale;
  double dx,x,y;
  HBRUSH hBrush;
  struct { int i; } cur;
  struct { double dy; } base;
  struct { double dy; } incr;
  int i;
  PAINTSTRUCT ps;
  HDC hDc;

  GetClientRect(hWnd,&rect);
  dx=(double)(rect.right-rect.left)/yas->hist.nbins;
  y=rect.bottom-rect.top;

  cur.i=yas_histogram_index(&yas->hist,yas->shuffle[yas->cur],yas->size);

  hDc=BeginPaint(hWnd,&ps); // {

  for (i=0,x=0.0;i<yas->hist.nbins;++i,x+=dx) {
    if (cur.i==i) {
      scale=yas->hist.bin[i].base+yas->hist.bin[i].incr;
      scale/=yas->hist.max;
      base.dy=scale*y;

      rect.left=x;
      rect.top=y-base.dy;
      rect.right=x+dx;
      rect.bottom=y;
      hBrush=CreateSolidBrush(RGB(204,196,0));

      FillRect(
        hDc,      // HDC        hDC,
        &rect,    // const RECT *lprc,
        hBrush    // HBRUSH     hbr
      );
    }
#if defined (YAS_PICTURE_NOTIFY) // [
    else if (yas->variables.trans.histogram) {
#else // ] [
    else if (yas->variables.cons.histogram) {
#endif // ]
      scale=yas->hist.bin[i].base;
      scale/=yas->hist.max;
      base.dy=scale*y;

      rect.left=x;
      rect.top=y-base.dy;
      rect.right=x+dx;
      rect.bottom=y;
      hBrush=CreateSolidBrush(RGB(0,208,39));

      FillRect(
        hDc,      // HDC        hDC,
        &rect,    // const RECT *lprc,
        hBrush    // HBRUSH     hbr
      );

      scale=yas->hist.bin[i].incr;
      scale/=yas->hist.max;
      incr.dy=scale*y;

      rect.left=x;
      rect.top=y-base.dy-incr.dy;
      rect.right=x+dx;
      rect.bottom=y-base.dy;
      hBrush=CreateSolidBrush(RGB(204,196,0));

      FillRect(
        hDc,      // HDC        hDC,
        &rect,    // const RECT *lprc,
        hBrush    // HBRUSH     hbr
      );
#if ! defined (YAS_PICTURE_NOTIFY) // [
    }
#else // ] [
    }
#endif // ]
    else {
      scale=yas->hist.bin[i].base+yas->hist.bin[i].incr;
      scale/=yas->hist.max;
      base.dy=scale*y;

      rect.left=x;
      rect.top=y-base.dy;
      rect.right=x+dx;
      rect.bottom=y;
      hBrush=CreateSolidBrush(RGB(0,208,39));

      FillRect(
        hDc,      // HDC        hDC,
        &rect,    // const RECT *lprc,
        hBrush    // HBRUSH     hbr
      );
    }
  }

  EndPaint(hWnd,&ps); // }
}
#else // ] [
static int picture_region_or_rect(HRGN *hRgn, int x1, int y1, int x2, int y2)
{
  HRGN hRgnRect;

  if (*hRgn) {
    hRgnRect=CreateRectRgn(
      x1,                   // int x1,
      y1,                   // int y1,
      x2,                   // int x2,
      y2                    // int y2
    );

    if (!hRgnRect) {
      DMESSAGE("creating region");
      goto e_region;
    }

    CombineRgn(
      *hRgn,                // HRGN hRgnDst,
      *hRgn,                // HRGN hRgnSrc1,
      hRgnRect,             // HRGN hRgnSrc2,
      RGN_OR                // int  iMode
    );

    DeleteObject(
      hRgnRect              // HGDIOBJ ho
    );
  }
  else {
    *hRgn=CreateRectRgn(
      x1,                   // int x1,
      y1,                   // int y1,
      x2,                   // int x2,
      y2                    // int y2
    );

    if (!*hRgn) {
      DMESSAGE("creating region");
      goto e_region;
    }
  }

  return 0;
e_region:
  return -1;
}

static void yas_image_callback(PibControl2 *control)
{
  Yas *yas=(Yas *)control->model;
  HWND hWnd=control->hWnd;
  HRGN hRgnBase=NULL;
  HRGN hRgnHighlight=NULL;
  RECT rect;
  double scale;
  double dx,x,y;
  HBRUSH hBrush;
  struct { int i; } cur;
  struct { double dy; } base;
  struct { double dy; } incr;
  int i;
  PAINTSTRUCT ps;
  HDC hDc;
  int err;

  GetClientRect(hWnd,&rect);
  dx=(double)(rect.right-rect.left)/yas->hist.nbins;
  y=rect.bottom-rect.top;

  cur.i=yas_histogram_index(&yas->hist,yas->shuffle[yas->cur],yas->size);
DVWRITELNW("### cur.i:%d",cur.i);

  for (i=0,x=0.0;i<yas->hist.nbins;++i,x+=dx) {
    if (cur.i==i) {
      scale=yas->hist.bin[i].base+yas->hist.bin[i].incr;
      scale/=yas->hist.max;
      base.dy=scale*y;

#if defined (YAS_PICTURE_ROUND) // [
      err=picture_region_or_rect(
        &hRgnHighlight,             // HRGN *hRgn,
        x+0.5,                      // int x1,
        (y-base.dy)+0.5,            // int y1,
        (x+dx)+0.5,                 // int x2,
        y+0.5                       // int y2
      );
#else // ] [
DVWRITELNW("### x1:%.0f y1:%.0f x2:%.0f y2:%.0f",
        x,                          // int x1,
        y-base.dy,                  // int y1,
        x+dx,                       // int x2,
        y                           // int y2
      );
      err=picture_region_or_rect(
        &hRgnHighlight,             // HRGN *hRgn,
        x,                          // int x1,
        y-base.dy,                  // int y1,
        x+dx,                       // int x2,
        y                           // int y2
      );
#endif // ]

      if (err<0)
        goto e_region;
    }
#if defined (YAS_PICTURE_NOTIFY) // [
    else if (yas->variables.trans.histogram) {
#else // ] [
    else if (yas->variables.cons.histogram) {
#endif // ]
      scale=yas->hist.bin[i].base;
      scale/=yas->hist.max;
      base.dy=scale*y;

#if defined (YAS_PICTURE_ROUND) // [
      err=picture_region_or_rect(
        &hRgnBase,                  // HRGN *hRgn,
        x+0.5,                      // int x1,
        (y-base.dy)+0.5,            // int y1,
        (x+dx)+0.5,                 // int x2,
        y+0.5                       // int y2
      );
#else // ] [
      err=picture_region_or_rect(
        &hRgnBase,                  // HRGN *hRgn,
        x,                          // int x1,
        y-base.dy,                  // int y1,
        x+dx,                       // int x2,
        y                           // int y2
      );
#endif // ]

      if (err<0)
        goto e_region;

      scale=yas->hist.bin[i].incr;
      scale/=yas->hist.max;
      incr.dy=scale*y;

#if defined (YAS_PICTURE_ROUND) // [
      err=picture_region_or_rect(
          &hRgnHighlight,           // HRGN *hRgn,
          x+0.5,                    // int x1,
          (y-base.dy-incr.dy)+0.5,  // int y1,
          (x+dx)+0.5,               // int x2,
          y-base.dy+0.5             // int y2
      );
#else // ] [
      err=picture_region_or_rect(
          &hRgnHighlight,           // HRGN *hRgn,
          x,                        // int x1,
          y-base.dy-incr.dy,        // int y1,
          x+dx,                     // int x2,
          y-base.dy                 // int y2
      );
#endif // ]

      if (err<0)
        goto e_region;
#if ! defined (YAS_PICTURE_NOTIFY) // [
    }
#else // ] [
    }
#endif // ]
    else {
      scale=yas->hist.bin[i].base+yas->hist.bin[i].incr;
      scale/=yas->hist.max;
      base.dy=scale*y;

#if defined (YAS_PICTURE_ROUND) // [
      err=picture_region_or_rect(
        &hRgnBase,                  // HRGN *hRgn,
        x+0.5,                      // int x1,
        (y-base.dy)+0.5,            // int y1,
        (x+dx)+0.5,                 // int x2,
        y+0.5                       // int y2
      );
#else // ] [
      err=picture_region_or_rect(
        &hRgnBase,                  // HRGN *hRgn,
        x,                          // int x1,
        y-base.dy,                  // int y1,
        x+dx,                       // int x2,
        y                           // int y2
      );
#endif // ]

      if (err<0)
        goto e_region;
    }
  }

  hDc=BeginPaint(hWnd,&ps); // {

  // All painting occurs here, between BeginPaint and EndPaint.
  if (hRgnBase) {
#if defined (DIFF_BASE) // [
    if (hRgnHighlight) {
      CombineRgn(
        hRgnBase,                   // HRGN hRgnDst,
        hRgnBase,                   // HRGN hRgnSrc1,
        hRgnHighlight,              // HRGN hRgnSrc2,
        RGN_DIFF                    // int  iMode
      );
    }
#endif // ]

    hBrush=CreateSolidBrush(RGB(0,208,39));

    FillRgn(
      hDc,                          // HDC    hdc,
      hRgnBase,                     // HRGN   hrgn,
      hBrush                        // HBRUSH hbr
    );
  }
  else {
    DVMESSAGE("something went wrong: %i: %lu %lu %lu %lu",
        cur.i,yas->cur,yas->shuffle[yas->cur],yas->size);
  }

  if (hRgnHighlight) {
DWRITELN("**************************************************");
    hBrush=CreateSolidBrush(RGB(204,196,0));

    FillRgn(
      hDc,                          // HDC    hdc,
      hRgnHighlight,                // HRGN   hrgn,
      hBrush                        // HBRUSH hbr
    );
  }

  EndPaint(hWnd,&ps); // }
e_region:
  if (hRgnBase) {
    DeleteObject(
      hRgnBase                      // HGDIOBJ ho
    );
  }

  if (hRgnHighlight) {
    DeleteObject(
      hRgnHighlight                 // HGDIOBJ ho
    );
  }
}
#endif // ]

/////////////////////////////////////////////////////////////////////////// } {
static void yas_label_callback(PibControl2 *control)
{
  enum { SIZE=256 };
  Yas *yas=(Yas *)control->model;
  double cur=yas->cur;
  double max=yas->size;
  wchar_t buf[SIZE];

  if (max) {
    /*
     * Unicode Character “≈” (U+2248)
     *   https://www.compart.com/en/unicode/U+2248
     */
    swprintf(buf,SIZE,L"%.0f/%.0f \x2248 %.1f%%",cur,max,100.0f*cur/max+0.5);
  }
  else
    buf[0]=L'\0';

  SetWindowTextW(control->hWnd,buf);
}

/////////////////////////////////////////////////////////////////////////// } {
static void yas_form_refresh(PibControl2 *control)
{
  PibControl2 *form=((Yas *)control->model)->form;

DWRITELN("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
  if (form)
    form->vmt->refresh(form);
}

/////////////////////////////////////////////////////////////////////////// } {
static void yas_progress_callback(PibControl2 *control)
{
  Yas *yas=(Yas *)control->model;
  double cur=yas->cur;
  double max=yas->size;
  double ratio=max?100.0*cur/max+0.5:0.0;

  SendMessageW(control->hWnd,PBM_SETPOS,ratio,0);
}

///////////////////////////////////////////////////////////////////////////// }
static PibControlParam2 yas_monitor_histogram
  =PIB_IMAGE(IDC_HISTOGRAM,yas_image_callback);
static PibControlParam2 yas_monitor_highlight
  =PIB_CHECK(IDC_CHECK_BOX_HISTOGRAM,YAS_PROPERTY_HISTOGRAM,yas_form_refresh);
static PibControlParam2 yas_monitor_progress_state
  =PIB_PROGRESS(IDC_PROGRESS_BAR_STATE,yas_progress_callback);
static PibControlParam2 yas_monitor_label_state
  =PIB_LABEL(IDC_STATIC_STATE,yas_label_callback);

static PibControlParam2 *yas_monitor_controls[]={
    &yas_monitor_histogram,
    &yas_monitor_highlight,
    &yas_monitor_progress_state,
    &yas_monitor_label_state,
};

PibControlParam2 yas_tabpage_monitor=PIB_TABPAGE(IDD_PAGE_MONITOR,
    L"Monitor",L"gen_yas.dll",yas_monitor_controls);
